@@ -1,5 +1,5 @@
 //
-// "$Id: Fl_Menu_Type.cxx 4727 2005-12-30 14:11:04Z matt $"
+// "$Id: Fl_Menu_Type.cxx 9082 2011-09-28 20:43:05Z matt $"
 //
 // Menu item code for the Fast Light Tool Kit (FLTK).
 //
@@ -9,22 +9,13 @@
 // This file also contains code to make Fl_Menu_Button, Fl_Menu_Bar,
 // etc widgets.
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -37,6 +28,8 @@
 #include <FL/fl_message.H>
 #include <FL/Fl_Menu_.H>
 #include <FL/Fl_Button.H>
+#include <FL/Fl_Value_Input.H>
+#include <FL/Fl_Text_Display.H>
 #include "../src/flstring.h"
 #include <stdio.h>
 #include <stdlib.h>
@@ -110,10 +103,10 @@ Fl_Type *Fl_Menu_Item_Type::make() {
   Fl_Type* q = Fl_Type::current;
   Fl_Type* p = q;
   if (p) {
-    if (force_parent && q->is_menu_item() || !q->is_parent()) p = p->parent;
+    if ( (force_parent && q->is_menu_item()) || !q->is_parent()) p = p->parent;
   }
   force_parent = 0;
-  if (!p || !(p->is_menu_button() || p->is_menu_item() && p->is_parent())) {
+  if (!p || !(p->is_menu_button() || (p->is_menu_item() && p->is_parent()))) {
     fl_message("Please select a menu to add to");
     return 0;
   }
@@ -168,7 +161,7 @@ const char* Fl_Menu_Item_Type::menu_name(int& i) {
 #include "Fluid_Image.h"
 
 void Fl_Menu_Item_Type::write_static() {
-  if (callback() && is_name(callback()))
+  if (callback() && is_name(callback()) && !user_defined(callback()))
     write_declare("extern void %s(Fl_Menu_*, %s);", callback(),
                   user_data_type() ? user_data_type() : "void*");
   for (int n=0; n < NUM_EXTRA_CODE; n++) {
@@ -209,10 +202,17 @@ void Fl_Menu_Item_Type::write_static() {
     write_c("\n}\n");
     if (k) {
       write_c("void %s::%s(Fl_Menu_* o, %s v) {\n", k, cn, ut);
-      write_c("  ((%s*)(o->", k);
+      write_c("  ((%s*)(o", k);
       Fl_Type* t = parent; while (t->is_menu_item()) t = t->parent;
-      for (t = t->parent; t && t->is_widget() && !is_class(); t = t->parent) write_c("parent()->");
-      write_c("user_data()))->%s_i(o,v);\n}\n", cn);
+      Fl_Type *q = 0;
+      // Go up one more level for Fl_Input_Choice, as these are groups themselves
+      if (t && !strcmp(t->type_name(), "Fl_Input_Choice"))
+        write_c("->parent()");
+      for (t = t->parent; t && t->is_widget() && !is_class(); q = t, t = t->parent) 
+        write_c("->parent()");
+      if (!q || strcmp(q->type_name(), "widget_class"))
+        write_c("->user_data()");
+      write_c("))->%s_i(o,v);\n}\n", cn);
     }
   }
   if (image) {
@@ -226,9 +226,13 @@ void Fl_Menu_Item_Type::write_static() {
   // entire array out:
   const char* k = class_name(1);
   if (k) {
-    int i; write_c("\nFl_Menu_Item %s::%s[] = {\n", k, menu_name(i));
+    int i; 
+    if (i18n_type) write_c("\nunsigned char %s::%s_i18n_done = 0;", k, menu_name(i));
+    write_c("\nFl_Menu_Item %s::%s[] = {\n", k, menu_name(i));
   } else {
-    int i; write_c("\nFl_Menu_Item %s[] = {\n", menu_name(i));
+    int i; 
+    if (i18n_type) write_c("\nunsigned char %s_i18n_done = 0;", menu_name(i));
+    write_c("\nFl_Menu_Item %s[] = {\n", menu_name(i));
   }
   Fl_Type* t = prev; while (t && t->is_menu_item()) t = t->prev;
   for (Fl_Type* q = t->next; q && q->is_menu_item(); q = q->next) {
@@ -244,10 +248,19 @@ void Fl_Menu_Item_Type::write_static() {
     // Write menu item variables...
     t = prev; while (t && t->is_menu_item()) t = t->prev;
     for (Fl_Type* q = t->next; q && q->is_menu_item(); q = q->next) {
-      const char *c = array_name((Fl_Menu_Item_Type *)q);
+      Fl_Menu_Item_Type *m = (Fl_Menu_Item_Type*)q;
+      const char *c = array_name(m);
       if (c) {
-      int i; const char* n = ((Fl_Menu_Item_Type *)q)->menu_name(i);
-      write_c("Fl_Menu_Item* %s::%s = %s::%s + %d;\n", k, c, k, n, i);
+        if (c==m->name()) {
+          // assign a menu item address directly to a variable
+          int i; 
+          const char* n = ((Fl_Menu_Item_Type *)q)->menu_name(i);
+          write_c("Fl_Menu_Item* %s::%s = %s::%s + %d;\n", k, c, k, n, i);
+        } else {
+          // if the name is an array, only define the array. 
+          // The actual assignment is in write_code1()
+          write_c("Fl_Menu_Item* %s::%s;\n", k, c);
+        }
       }
     }
   }
@@ -280,28 +293,16 @@ void Fl_Menu_Item_Type::write_item() {
 
   write_c(" {");
   if (image) write_c("0");
-  else if (label()) {
-    switch (i18n_type) {
-    case 0 : /* None */
-        write_cstring(label());
-        break;
-    case 1 : /* GNU gettext */
-        write_c("%s(", i18n_function);
-        write_cstring(label());
-	write_c(")");
-        break;
-    case 2 : /* POSIX catgets */
-        write_c("catgets(%s,%s,%d,", i18n_file[0] ? i18n_file : "_catalog",
-	        i18n_set, msgnum());
-        write_cstring(label());
-	write_c(")");
-        break;
-    }
-  }
+  else if (label()) write_cstring(label()); // we will call i18n when the widget is instantiated for the first time
   else write_c("\"\"");
-  if (((Fl_Button*)o)->shortcut())
-    write_c(", 0x%x, ", ((Fl_Button*)o)->shortcut());
-  else
+  if (((Fl_Button*)o)->shortcut()) {
+		int s = ((Fl_Button*)o)->shortcut();
+		if (use_FL_COMMAND && (s & (FL_CTRL|FL_META))) {
+			write_c(", FL_COMMAND|0x%x, ", s & ~(FL_CTRL|FL_META));
+		} else {
+			write_c(", 0x%x, ", s);
+		}
+  } else
     write_c(", 0, ");
   if (callback()) {
     const char* k = is_name(callback()) ? 0 : class_name(1);
@@ -325,10 +326,13 @@ void Fl_Menu_Item_Type::write_code1() {
   int i; const char* mname = menu_name(i);
   if (!prev->is_menu_item()) {
     // for first menu item, declare the array
-    if (class_name(1))
+    if (class_name(1)) {
+      if (i18n_type) write_h("  static unsigned char %s_i18n_done;\n", mname);
       write_h("  static Fl_Menu_Item %s[];\n", mname);
-    else
+    } else {
+      if (i18n_type) write_h("extern unsigned char %s_i18n_done;\n", mname);
       write_h("extern Fl_Menu_Item %s[];\n", mname);
+    }
   }
 
   const char *c = array_name(this);
@@ -336,8 +340,12 @@ void Fl_Menu_Item_Type::write_code1() {
     if (class_name(1)) {
       write_public(public_);
       write_h("  static Fl_Menu_Item *%s;\n", c);
-    } else
-      write_h("#define %s (%s+%d)\n", c, mname, i);
+    } else {
+      if (c==name())
+        write_h("#define %s (%s+%d)\n", c, mname, i);
+      else
+        write_h("extern Fl_Menu_Item *%s;\n", c);
+    }
   }
 
   if (callback()) {
@@ -345,16 +353,22 @@ void Fl_Menu_Item_Type::write_code1() {
       const char* cn = callback_name();
       const char* ut = user_data_type() ? user_data_type() : "void*";
       write_public(0);
-      write_h("  void %s_i(Fl_Menu_*, %s);\n", cn, ut);
+      write_h("  inline void %s_i(Fl_Menu_*, %s);\n", cn, ut);
       write_h("  static void %s(Fl_Menu_*, %s);\n", cn, ut);
     }
   }
 
   int init = 0;
+  // if the name is an array variable, assign the value here
+  if (name() && strchr(name(), '[')) {
+    write_c("%s%s = &%s[%d];\n", indent(), name(), mname, i);
+  }
   if (image) {
-    write_c(" {Fl_Menu_Item* o = &%s[%d];\n", mname, i);
-    init = 1;
-    image->write_code();
+    if (!init) {
+      init = 1;
+      write_c("%s{ Fl_Menu_Item* o = &%s[%d];\n", indent(), mname, i);
+    }
+    image->write_code("o");
   }
   for (int n=0; n < NUM_EXTRA_CODE; n++)
     if (extra_code(n) && !isdeclare(extra_code(n))) {
@@ -442,9 +456,42 @@ Fl_Type* Fl_Menu_Type::click_test(int, int) {
 }
 
 void Fl_Menu_Type::write_code2() {
-  if (next && next->is_menu_item())
-    write_c("%so->menu(%s);\n", indent(),
+  if (next && next->is_menu_item()) {
+    if (i18n_type) {
+      // take care of i18n now!
+      Fl_Menu_Item_Type *mi = (Fl_Menu_Item_Type*)next;
+      int i, nItem = 0, nLabel = 0;
+      const char *mName = mi->menu_name(i);
+      for (Fl_Type* q = next; q && q->is_menu_item(); q = q->next) {
+        if (((Fl_Menu_Item_Type*)q)->label()) nLabel++;
+	int thislevel = q->level; if (q->is_parent()) thislevel++;
+	int nextlevel =
+	    (q->next && q->next->is_menu_item()) ? q->next->level : next->level+1;
+	nItem += 1 + ((thislevel > nextlevel) ? (thislevel-nextlevel) : 0);
+      }
+      if (nLabel) {
+        write_c("%sif (!%s_i18n_done) {\n", indent(), mName);
+        write_c("%s  int i=0;\n", indent());
+        write_c("%s  for ( ; i<%d; i++)\n", indent(), nItem);
+        write_c("%s    if (%s[i].label())\n", indent(), mName);
+        switch (i18n_type) {
+          case 1:
+            write_c("%s      %s[i].label(%s(%s[i].label()));\n",
+                    indent(), mName, i18n_function, mName);
+            break;
+          case 2:
+            write_c("%s      %s[i].label(catgets(%s,%s,i+%d,%s[i].label()));\n",
+                    indent(), mName, i18n_file[0] ? i18n_file : "_catalog", 
+                    i18n_set, mi->msgnum(), mName);
+            break;
+        }
+        write_c("%s  %s_i18n_done = 1;\n", indent(), mName);
+        write_c("%s}\n", indent());
+      }
+    }
+    write_c("%s%s->menu(%s);\n", indent(), name() ? name() : "o",
 	    unique_id(this, "menu", name(), label()));
+  }
   Fl_Widget_Type::write_code2();
 }
 
@@ -525,7 +572,13 @@ void Shortcut_Button::draw() {
   if (value()) draw_box(FL_DOWN_BOX, (Fl_Color)9);
   else draw_box(FL_UP_BOX, FL_WHITE);
   fl_font(FL_HELVETICA,14); fl_color(FL_FOREGROUND_COLOR);
-  fl_draw(fl_shortcut_label(svalue),x()+6,y(),w(),h(),FL_ALIGN_LEFT);
+	if (use_FL_COMMAND && (svalue & (FL_CTRL|FL_META))) {
+		char buf[1024];
+		fl_snprintf(buf, 1023, "Command+%s", fl_shortcut_label(svalue&~(FL_CTRL|FL_META)));
+		fl_draw(buf,x()+6,y(),w(),h(),FL_ALIGN_LEFT);
+	} else {
+		fl_draw(fl_shortcut_label(svalue),x()+6,y(),w(),h(),FL_ALIGN_LEFT);
+	}
 }
 
 int Shortcut_Button::handle(int e) {
@@ -533,14 +586,14 @@ int Shortcut_Button::handle(int e) {
   if (e == FL_KEYBOARD) {
     if (!value()) return 0;
     int v = Fl::event_text()[0];
-    if (v > 32 && v < 0x7f || v > 0xa0 && v <= 0xff) {
+    if ( (v > 32 && v < 0x7f) || (v > 0xa0 && v <= 0xff) ) {
       if (isupper(v)) {
         v = tolower(v);
         v |= FL_SHIFT;
       }
-      v = v | Fl::event_state()&(FL_META|FL_ALT|FL_CTRL);
+      v = v | (Fl::event_state()&(FL_META|FL_ALT|FL_CTRL));
     } else {
-      v = Fl::event_state()&(FL_META|FL_ALT|FL_CTRL|FL_SHIFT) | Fl::event_key();
+      v = (Fl::event_state()&(FL_META|FL_ALT|FL_CTRL|FL_SHIFT)) | Fl::event_key();
       if (v == FL_BackSpace && svalue) v = 0;
     }
     if (v != svalue) {svalue = v; set_changed(); redraw(); do_callback(); }
@@ -559,20 +612,45 @@ int Shortcut_Button::handle(int e) {
   
 void shortcut_in_cb(Shortcut_Button* i, void* v) {
   if (v == LOAD) {
-    if (!current_widget->is_button()) {i->hide(); return;}
+    if (current_widget->is_button())
+      i->svalue = ((Fl_Button*)(current_widget->o))->shortcut();
+    else if (current_widget->is_input())
+      i->svalue = ((Fl_Input_*)(current_widget->o))->shortcut();
+    else if (current_widget->is_value_input())
+      i->svalue = ((Fl_Value_Input*)(current_widget->o))->shortcut();
+    else if (current_widget->is_text_display())
+      i->svalue = ((Fl_Text_Display*)(current_widget->o))->shortcut();
+    else {
+      i->hide();
+      return;
+    }
     i->show();
-    i->svalue = ((Fl_Button*)(current_widget->o))->shortcut();
     i->redraw();
   } else {
+    int mod = 0;
     for (Fl_Type *o = Fl_Type::first; o; o = o->next)
       if (o->selected && o->is_button()) {
 	Fl_Button* b = (Fl_Button*)(((Fl_Widget_Type*)o)->o);
+        if (b->shortcut()!=i->svalue) mod = 1;
 	b->shortcut(i->svalue);
 	if (o->is_menu_item()) ((Fl_Widget_Type*)o)->redraw();
+      } else if (o->selected && o->is_input()) {
+	Fl_Input_* b = (Fl_Input_*)(((Fl_Widget_Type*)o)->o);
+        if (b->shortcut()!=i->svalue) mod = 1;
+	b->shortcut(i->svalue);
+      } else if (o->selected && o->is_value_input()) {
+	Fl_Value_Input* b = (Fl_Value_Input*)(((Fl_Widget_Type*)o)->o);
+        if (b->shortcut()!=i->svalue) mod = 1;
+	b->shortcut(i->svalue);
+      } else if (o->selected && o->is_text_display()) {
+	Fl_Text_Display* b = (Fl_Text_Display*)(((Fl_Widget_Type*)o)->o);
+        if (b->shortcut()!=i->svalue) mod = 1;
+	b->shortcut(i->svalue);
       }
+    if (mod) set_modflag(1);
   }
 }
 
 //
-// End of "$Id: Fl_Menu_Type.cxx 4727 2005-12-30 14:11:04Z matt $".
+// End of "$Id: Fl_Menu_Type.cxx 9082 2011-09-28 20:43:05Z matt $".
 //
