@@ -1,24 +1,15 @@
 //
-// "$Id: Fl_Color_Chooser.cxx 5190 2006-06-09 16:16:34Z mike $"
+// "$Id: Fl_Color_Chooser.cxx 9673 2012-08-18 10:47:48Z AlbrechtS $"
 //
 // Color chooser for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -47,6 +38,11 @@
 // you get this by defining this:
 #define UPDATE_HUE_BOX 1
 
+/**
+  This \e static method converts HSV colors to RGB colorspace.
+  \param[in] H, S, V color components
+  \param[out] R, G, B color components
+ */
 void Fl_Color_Chooser::hsv2rgb(
 	double H, double S, double V, double& R, double& G, double& B) {
   if (S < 5.0e-6) {
@@ -68,6 +64,11 @@ void Fl_Color_Chooser::hsv2rgb(
   }
 }
 
+/**
+  This \e static method converts RGB colors to HSV colorspace.
+  \param[in] R, G, B color components
+  \param[out] H, S, V color components
+ */
 void Fl_Color_Chooser::rgb2hsv(
 	double R, double G, double B, double& H, double& S, double& V) {
   double maxv = R > G ? R : G; if (B > maxv) maxv = B;
@@ -83,7 +84,13 @@ void Fl_Color_Chooser::rgb2hsv(
   }
 }
 
-enum {M_RGB, M_BYTE, M_HEX, M_HSV}; // modes
+/** Fl_Color_Chooser modes */
+enum {
+  M_RGB,	/**< mode() of Fl_Color_Chooser showing RGB values */
+  M_BYTE,	/**< mode() of Fl_Color_Chooser showing byte values */
+  M_HEX,	/**< mode() of Fl_Color_Chooser showing hex values */
+  M_HSV		/**< mode() of Fl_Color_Chooser showing HSV values */
+};
 static Fl_Menu_Item mode_menu[] = {
   {"rgb"},
   {"byte"},
@@ -92,11 +99,13 @@ static Fl_Menu_Item mode_menu[] = {
   {0}
 };
 
+#ifndef FL_DOXYGEN
 int Flcc_Value_Input::format(char* buf) {
   Fl_Color_Chooser* c = (Fl_Color_Chooser*)parent();
   if (c->mode() == M_HEX) return sprintf(buf,"0x%02X", int(value()));
   else return Fl_Valuator::format(buf);
 }
+#endif // !FL_DOXYGEN
 
 void Fl_Color_Chooser::set_valuators() {
   switch (mode()) {
@@ -105,7 +114,7 @@ void Fl_Color_Chooser::set_valuators() {
     gvalue.range(0,1); gvalue.step(1,1000); gvalue.value(g_);
     bvalue.range(0,1); bvalue.step(1,1000); bvalue.value(b_);
     break;
-  case M_BYTE:
+  case M_BYTE: /* FALLTHROUGH */
   case M_HEX:
     rvalue.range(0,255); rvalue.step(1); rvalue.value(int(255*r_+.5));
     gvalue.range(0,255); gvalue.step(1); gvalue.value(int(255*g_+.5));
@@ -119,6 +128,13 @@ void Fl_Color_Chooser::set_valuators() {
   }
 }
 
+/**
+  Sets the current rgb color values.
+  Does not do the callback. Does not clamp (but out of range values will
+  produce psychedelic effects in the hue selector).
+  \param[in] R, G, B color components.
+  \return 1 if a new rgb value was set, 0 if the rgb value was the previous one.
+ */
 int Fl_Color_Chooser::rgb(double R, double G, double B) {
   if (R == r_ && G == g_ && B == b_) return 0;
   r_ = R; g_ = G; b_ = B;
@@ -140,6 +156,13 @@ int Fl_Color_Chooser::rgb(double R, double G, double B) {
   return 1;
 }
 
+/**
+  Set the hsv values.
+  The passed values are clamped (or for hue, modulus 6 is used) to get
+  legal values. Does not do the callback.
+  \param[in] H, S, V color components.
+  \return 1 if a new hsv value was set, 0 if the hsv value was the previous one.
+*/
 int Fl_Color_Chooser::hsv(double H, double S, double V) {
   H = fmod(H,6.0); if (H < 0.0) H += 6.0;
   if (S < 0.0) S = 0.0; else if (S > 1.0) S = 1.0;
@@ -179,6 +202,7 @@ static void tohs(double x, double y, double& h, double& s) {
 #endif
 }
 
+#ifndef FL_DOXYGEN
 int Flcc_HueBox::handle(int e) {
   static double ih, is;
   Fl_Color_Chooser* c = (Fl_Color_Chooser*)parent();
@@ -200,7 +224,7 @@ int Flcc_HueBox::handle(int e) {
     if (Fl::event_state(FL_CTRL)) H = ih;
     if (c->hsv(H, S, c->value())) c->do_callback();
     } return 1;
-  case FL_FOCUS :
+  case FL_FOCUS : /* FALLTHROUGH */
   case FL_UNFOCUS :
     if (Fl::visible_focus()) {
       redraw();
@@ -213,6 +237,7 @@ int Flcc_HueBox::handle(int e) {
     return 0;
   }
 }
+#endif // !FL_DOXYGEN
 
 static void generate_image(void* vv, int X, int Y, int W, uchar* buf) {
   Flcc_HueBox* v = (Flcc_HueBox*)vv;
@@ -226,7 +251,7 @@ static void generate_image(void* vv, int X, int Y, int W, uchar* buf) {
   for (int x = X; x < X+W; x++) {
     double Xf = double(x)/iw;
     double H,S; tohs(Xf,Yf,H,S);
-    double r,g,b;
+    double r=0, g=0, b=0;
     Fl_Color_Chooser::hsv2rgb(H,S,V,r,g,b);
     *buf++ = uchar(255*r+.5);
     *buf++ = uchar(255*g+.5);
@@ -234,6 +259,7 @@ static void generate_image(void* vv, int X, int Y, int W, uchar* buf) {
   }
 }
 
+#ifndef FL_DOXYGEN
 int Flcc_HueBox::handle_key(int key) {
   int w1 = w()-Fl::box_dw(box())-6;
   int h1 = h()-Fl::box_dh(box())-6;
@@ -272,14 +298,16 @@ int Flcc_HueBox::handle_key(int key) {
 
   return 1;
 }
+#endif // !FL_DOXYGEN
 
+#ifndef FL_DOXYGEN
 void Flcc_HueBox::draw() {
   if (damage()&FL_DAMAGE_ALL) draw_box();
   int x1 = x()+Fl::box_dx(box());
   int yy1 = y()+Fl::box_dy(box());
   int w1 = w()-Fl::box_dw(box());
   int h1 = h()-Fl::box_dh(box());
-  if (damage() == FL_DAMAGE_EXPOSE) fl_clip(x1+px,yy1+py,6,6);
+  if (damage() == FL_DAMAGE_EXPOSE) fl_push_clip(x1+px,yy1+py,6,6);
   fl_draw_image(generate_image, this, x1, yy1, w1, h1);
   if (damage() == FL_DAMAGE_EXPOSE) fl_pop_clip();
   Fl_Color_Chooser* c = (Fl_Color_Chooser*)parent();
@@ -296,9 +324,11 @@ void Flcc_HueBox::draw() {
   draw_box(FL_UP_BOX,x1+X,yy1+Y,6,6,Fl::focus() == this ? FL_FOREGROUND_COLOR : FL_GRAY);
   px = X; py = Y;
 }
+#endif // !FL_DOXYGEN
 
 ////////////////////////////////////////////////////////////////
 
+#ifndef FL_DOXYGEN
 int Flcc_ValueBox::handle(int e) {
   static double iv;
   Fl_Color_Chooser* c = (Fl_Color_Chooser*)parent();
@@ -315,7 +345,7 @@ int Flcc_ValueBox::handle(int e) {
     if (fabs(Yf-iv)<(3*1.0/h())) Yf = iv;
     if (c->hsv(c->hue(),c->saturation(),Yf)) c->do_callback();
     } return 1;
-  case FL_FOCUS :
+  case FL_FOCUS : /* FALLTHROUGH */
   case FL_UNFOCUS :
     if (Fl::visible_focus()) {
       redraw();
@@ -328,6 +358,7 @@ int Flcc_ValueBox::handle(int e) {
     return 0;
   }
 }
+#endif // !FL_DOXYGEN
 
 static double tr, tg, tb;
 static void generate_vimage(void* vv, int X, int Y, int W, uchar* buf) {
@@ -341,6 +372,7 @@ static void generate_vimage(void* vv, int X, int Y, int W, uchar* buf) {
   }
 }
 
+#ifndef FL_DOXYGEN
 void Flcc_ValueBox::draw() {
   if (damage()&FL_DAMAGE_ALL) draw_box();
   Fl_Color_Chooser* c = (Fl_Color_Chooser*)parent();
@@ -349,7 +381,7 @@ void Flcc_ValueBox::draw() {
   int yy1 = y()+Fl::box_dy(box());
   int w1 = w()-Fl::box_dw(box());
   int h1 = h()-Fl::box_dh(box());
-  if (damage() == FL_DAMAGE_EXPOSE) fl_clip(x1,yy1+py,w1,6);
+  if (damage() == FL_DAMAGE_EXPOSE) fl_push_clip(x1,yy1+py,w1,6);
   fl_draw_image(generate_vimage, this, x1, yy1, w1, h1);
   if (damage() == FL_DAMAGE_EXPOSE) fl_pop_clip();
   int Y = int((1-c->value()) * (h1-6));
@@ -357,7 +389,9 @@ void Flcc_ValueBox::draw() {
   draw_box(FL_UP_BOX,x1,yy1+Y,w1,6,Fl::focus() == this ? FL_FOREGROUND_COLOR : FL_GRAY);
   py = Y;
 }
+#endif // !FL_DOXYGEN
 
+#ifndef FL_DOXYGEN
 int Flcc_ValueBox::handle_key(int key) {
   int h1 = h()-Fl::box_dh(box())-6;
   Fl_Color_Chooser* c = (Fl_Color_Chooser*)parent();
@@ -382,6 +416,7 @@ int Flcc_ValueBox::handle_key(int key) {
 
   return 1;
 }
+#endif // !FL_DOXYGEN
 
 ////////////////////////////////////////////////////////////////
 
@@ -411,8 +446,22 @@ void Fl_Color_Chooser::mode_cb(Fl_Widget* o, void*) {
   c->set_valuators();
 }
 
+void Fl_Color_Chooser::mode(int newMode)
+{
+  choice.value(newMode);
+  choice.do_callback();
+}
+
+
 ////////////////////////////////////////////////////////////////
 
+/**
+  Creates a new Fl_Color_Chooser widget using the given position, size, and
+  label string.
+  The recommended dimensions are 200x95. The color is initialized to black.
+  \param[in] X, Y, W, H position and size of the widget
+  \param[in] L widget label, default is no label
+ */
 Fl_Color_Chooser::Fl_Color_Chooser(int X, int Y, int W, int H, const char* L)
   : Fl_Group(0,0,195,115,L),
     huebox(0,0,115,115),
@@ -465,7 +514,7 @@ void ColorChip::draw() {
 	   h()-Fl::box_dh(box()),r,g,b);
 }
 
-static void chooser_cb(Fl_Object* o, void* vv) {
+static void chooser_cb(Fl_Widget* o, void* vv) {
   Fl_Color_Chooser* c = (Fl_Color_Chooser*)o;
   ColorChip* v = (ColorChip*)vv;
   v->r = uchar(255*c->r()+.5);
@@ -477,45 +526,89 @@ static void chooser_cb(Fl_Object* o, void* vv) {
 extern const char* fl_ok;
 extern const char* fl_cancel;
 
-int fl_color_chooser(const char* name, double& r, double& g, double& b) {
+// fl_color_chooser's callback for ok_button (below)
+//  [in] o is a pointer to okay_button (below) 
+//  [in] p is a pointer to an int to receive the return value (1)
+// closes the fl_color_chooser window
+static void cc_ok_cb (Fl_Widget *o, void *p) {
+  *((int *)p) = 1; // set return value
+  o->window()->hide();
+}
+
+// fl_color_chooser's callback for cancel_button and window close
+//  [in] o is a pointer to cancel_button (below) _or_ the dialog window
+//  [in] p is a pointer to an int to receive the return value (0)
+// closes the fl_color_chooser window
+static void cc_cancel_cb (Fl_Widget *o, void *p) {
+  *((int *)p) = 0; // set return value
+  if (o->window()) // cancel button
+    o->window()->hide();
+  else // window close
+    o->hide();
+}
+
+/** \addtogroup  group_comdlg 
+    @{ */
+/**
+  \brief Pops up a window to let the user pick an arbitrary RGB color.
+  \note \#include <FL/Fl_Color_Chooser.H>
+  \image html fl_color_chooser.jpg 
+  \image latex  fl_color_chooser.jpg "fl_color_chooser" width=8cm
+  \param[in] name Title label for the window
+  \param[in,out] r, g, b Color components in the range 0.0 to 1.0.
+  \param[in] cmode Optional mode for color chooser. See mode(int). Default -1 if none (rgb mode).
+  \retval 1 if user confirms the selection 
+  \retval 0 if user cancels the dialog
+  \relates Fl_Color_Chooser
+ */
+int fl_color_chooser(const char* name, double& r, double& g, double& b, int cmode) {
+  int ret = 0;
   Fl_Window window(215,200,name);
+  window.callback(cc_cancel_cb,&ret);
   Fl_Color_Chooser chooser(10, 10, 195, 115);
   ColorChip ok_color(10, 130, 95, 25);
   Fl_Return_Button ok_button(10, 165, 95, 25, fl_ok);
+  ok_button.callback(cc_ok_cb,&ret);
   ColorChip cancel_color(110, 130, 95, 25);
   cancel_color.r = uchar(255*r+.5); ok_color.r = cancel_color.r;
   ok_color.g = cancel_color.g = uchar(255*g+.5);
   ok_color.b = cancel_color.b = uchar(255*b+.5);
   Fl_Button cancel_button(110, 165, 95, 25, fl_cancel);
+  cancel_button.callback(cc_cancel_cb,&ret);
   window.resizable(chooser);
   chooser.rgb(r,g,b);
   chooser.callback(chooser_cb, &ok_color);
+  if (cmode!=-1) chooser.mode(cmode);
   window.end();
   window.set_modal();
   window.hotspot(window);
   window.show();
-  while (window.shown()) {
-    Fl::wait();
-    for (;;) {
-      Fl_Widget* o = Fl::readqueue();
-      if (!o) break;
-      if (o == &ok_button) {
-	r = chooser.r();
-	g = chooser.g();
-	b = chooser.b();
-	return 1;
-      }
-      if (o == &window || o == &cancel_button) return 0;
-    }
+  while (window.shown()) Fl::wait();
+  if (ret) { // ok_button or Enter
+    r = chooser.r();
+    g = chooser.g();
+    b = chooser.b();
   }
-  return 0;
+  return ret;
 }
 
-int fl_color_chooser(const char* name, uchar& r, uchar& g, uchar& b) {
+/**
+  \brief Pops up a window to let the user pick an arbitrary RGB color.
+  \note \#include <FL/Fl_Color_Chooser.H>
+  \image html fl_color_chooser.jpg 
+  \image latex  fl_color_chooser.jpg "fl_color_chooser" width=8cm
+  \param[in] name Title label for the window
+  \param[in,out] r, g, b Color components in the range 0 to 255.
+  \param[in] cmode Optional mode for color chooser. See mode(int). Default -1 if none (rgb mode).
+  \retval 1 if user confirms the selection 
+  \retval 0 if user cancels the dialog
+  \relates Fl_Color_Chooser
+ */
+int fl_color_chooser(const char* name, uchar& r, uchar& g, uchar& b, int cmode) {
   double dr = r/255.0;
   double dg = g/255.0;
   double db = b/255.0;
-  if (fl_color_chooser(name,dr,dg,db)) {
+  if (fl_color_chooser(name,dr,dg,db,cmode)) {
     r = uchar(255*dr+.5);
     g = uchar(255*dg+.5);
     b = uchar(255*db+.5);
@@ -524,6 +617,7 @@ int fl_color_chooser(const char* name, uchar& r, uchar& g, uchar& b) {
   return 0;
 }
 
+/** @} */
 //
-// End of "$Id: Fl_Color_Chooser.cxx 5190 2006-06-09 16:16:34Z mike $".
+// End of "$Id: Fl_Color_Chooser.cxx 9673 2012-08-18 10:47:48Z AlbrechtS $".
 //
