@@ -1,33 +1,29 @@
 //
-// "$Id: Fl_Double_Window.cxx 5829 2007-05-14 15:51:00Z matt $"
+// "$Id: Fl_Double_Window.cxx 9719 2012-11-13 14:45:42Z manolo $"
 //
 // Double-buffered window code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2007 by Bill Spitzak and others.
+// Copyright 1998-2012 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
 //     http://www.fltk.org/str.php
 //
+/** \file
+ Fl_Double_Window implementation.
+ */
 
 #include <config.h>
 #include <FL/Fl.H>
 #include <FL/Fl_Double_Window.H>
+#include <FL/Fl_Overlay_Window.H>
+#include <FL/Fl_Printer.H>
 #include <FL/x.H>
 #include <FL/fl_draw.H>
 
@@ -51,20 +47,98 @@ static int can_xdbe() {
     int numscreens = 1;
     XdbeScreenVisualInfo *a = XdbeGetVisualInfo(fl_display,&root,&numscreens);
     if (!a) return 0;
-    for (int j = 0; j < a->count; j++)
+    for (int j = 0; j < a->count; j++) {
       if (a->visinfo[j].visual == fl_visual->visualid
-	  /*&& a->visinfo[j].perflevel > 0*/) {use_xdbe = 1; break;}
+	  /*&& a->visinfo[j].perflevel > 0*/) {
+        use_xdbe = 1; break;
+      }
+    }
     XdbeFreeVisualInfo(a);
   }
   return use_xdbe;
 }
 #endif
 
+
+Fl_Double_Window::Fl_Double_Window(int W, int H, const char *l) 
+: Fl_Window(W,H,l), 
+  force_doublebuffering_(0) 
+{ 
+  type(FL_DOUBLE_WINDOW); 
+}
+
+
+Fl_Double_Window::Fl_Double_Window(int X, int Y, int W, int H, const char *l)
+: Fl_Window(X,Y,W,H,l), 
+  force_doublebuffering_(0) 
+{ 
+  type(FL_DOUBLE_WINDOW);
+}
+
+
 void Fl_Double_Window::show() {
   Fl_Window::show();
 }
 
+
+/** \addtogroup fl_drawings
+ @{
+ */
+/** Copy a rectangular area of the given offscreen buffer into the current drawing destination.
+ \param x,y	position where to draw the copied rectangle
+ \param w,h	size of the copied rectangle
+ \param pixmap  offscreen buffer containing the rectangle to copy
+ \param srcx,srcy origin in offscreen buffer of rectangle to copy
+ */
+#if FLTK_ABI_VERSION >= 10301
+void fl_copy_offscreen(int x, int y, int w, int h, Fl_Offscreen pixmap, int srcx, int srcy) {
+  fl_graphics_driver->copy_offscreen(x, y, w, h, pixmap, srcx, srcy);
+}
+#else
+void fl_copy_offscreen(int x, int y, int w, int h, Fl_Offscreen pixmap, int srcx, int srcy) {
 #ifdef WIN32
+  if (fl_graphics_driver->class_name() == Fl_GDI_Graphics_Driver::class_id ||
+      fl_graphics_driver->class_name() == Fl_GDI_Printer_Graphics_Driver::class_id) {
+#else
+  if (fl_graphics_driver->class_name() == Fl_Display_Device::display_device()->driver()->class_name()) {
+#endif
+#ifdef USE_X11
+    ((Fl_Xlib_Graphics_Driver*)fl_graphics_driver)->copy_offscreen(x, y, w, h, pixmap, srcx, srcy);
+#elif defined(WIN32)
+    ((Fl_GDI_Graphics_Driver*)fl_graphics_driver)->copy_offscreen(x, y, w, h, pixmap, srcx, srcy);
+#elif defined(__APPLE__)
+    ((Fl_Quartz_Graphics_Driver*)fl_graphics_driver)->copy_offscreen(x, y, w, h, pixmap, srcx, srcy);
+#endif
+  }
+  else { // when copy is not to the display
+    fl_graphics_driver->copy_offscreen(x, y, w, h, pixmap, srcx, srcy);
+  }
+}
+#endif // FLTK_ABI_VERSION
+/** @} */
+
+/** see fl_copy_offscreen() */
+void Fl_Graphics_Driver::copy_offscreen(int x, int y, int w, int h, Fl_Offscreen pixmap, int srcx, int srcy)
+{
+  fl_begin_offscreen(pixmap);
+  uchar *img = fl_read_image(NULL, srcx, srcy, w, h, 0);
+  fl_end_offscreen();
+  fl_draw_image(img, x, y, w, h, 3, 0);
+  delete[] img;
+}
+
+#if defined(USE_X11)
+
+void Fl_Xlib_Graphics_Driver::copy_offscreen(int x, int y, int w, int h, Fl_Offscreen pixmap, int srcx, int srcy) {
+  XCopyArea(fl_display, pixmap, fl_window, fl_gc, srcx, srcy, w, h, x, y);
+}
+
+
+// maybe someone feels inclined to implement alpha blending on X11?
+char fl_can_do_alpha_blending() {
+  return 0;
+}
+#elif defined(WIN32)
 
 // Code used to switch output to an off-screen window.  See macros in
 // win32.H which save the old state in local variables.
@@ -94,10 +168,9 @@ char fl_can_do_alpha_blending() {
   fl_alpha_blend = (fl_alpha_blend_func)GetProcAddress(hMod, "AlphaBlend");
   // give up if we can't find it (Win95)
   if (!fl_alpha_blend) return 0;
-  // we have the  call, but does our display support alpha blending?
-  HDC dc = 0L;//fl_gc;
-  // get the current or the desktop's device context
-  if (!dc) dc = GetDC(0L);
+  // we have the call, but does our display support alpha blending?
+  // get the desktop's device context
+  HDC dc = GetDC(0L);
   if (!dc) return 0;
   // check the device capabilities flags. However GetDeviceCaps
   // does not return anything useful, so we have to do it manually:
@@ -111,8 +184,8 @@ char fl_can_do_alpha_blending() {
   RestoreDC(new_gc, save);
   DeleteDC(new_gc);
   DeleteObject(bm);
+  ReleaseDC(0L, dc);
 
-  if (!fl_gc) ReleaseDC(0L, dc);
   if (alpha_ok) can_do = 1;
   return can_do;
 }
@@ -128,7 +201,7 @@ HDC fl_makeDC(HBITMAP bitmap) {
   return new_gc;
 }
 
-void fl_copy_offscreen(int x,int y,int w,int h,HBITMAP bitmap,int srcx,int srcy) {
+void Fl_GDI_Graphics_Driver::copy_offscreen(int x,int y,int w,int h,HBITMAP bitmap,int srcx,int srcy) {
   HDC new_gc = CreateCompatibleDC(fl_gc);
   int save = SaveDC(new_gc);
   SelectObject(new_gc, bitmap);
@@ -137,106 +210,53 @@ void fl_copy_offscreen(int x,int y,int w,int h,HBITMAP bitmap,int srcx,int srcy)
   DeleteDC(new_gc);
 }
 
-void fl_copy_offscreen_with_alpha(int x,int y,int w,int h,HBITMAP bitmap,int srcx,int srcy) {
+void Fl_GDI_Graphics_Driver::copy_offscreen_with_alpha(int x,int y,int w,int h,HBITMAP bitmap,int srcx,int srcy) {
   HDC new_gc = CreateCompatibleDC(fl_gc);
   int save = SaveDC(new_gc);
   SelectObject(new_gc, bitmap);
+  fl_can_do_alpha_blending(); // make sure this is called
   BOOL alpha_ok = 0;
   // first try to alpha blend
-  if (fl_can_do_alpha_blending())
-    alpha_ok = fl_alpha_blend(fl_gc, x, y, w, h, new_gc, srcx, srcy, w, h, blendfunc);
-  // if that failed (it shouldn,t), still copy the bitmap over, but now alpha is 1
-  if (!alpha_ok)
+  // if to printer, always try alpha_blend
+  if ( Fl_Surface_Device::surface() != Fl_Display_Device::display_device() || fl_can_do_alpha_blending() ) {
+    if (fl_alpha_blend) alpha_ok = fl_alpha_blend(fl_gc, x, y, w, h, new_gc, srcx, srcy, w, h, blendfunc);
+  }
+  // if that failed (it shouldn't), still copy the bitmap over, but now alpha is 1
+  if (!alpha_ok) {
     BitBlt(fl_gc, x, y, w, h, new_gc, srcx, srcy, SRCCOPY);
+  }
   RestoreDC(new_gc, save);
   DeleteDC(new_gc);
 }
 
 extern void fl_restore_clip();
 
-#elif defined(__APPLE_QD__)
-
-char fl_can_do_alpha_blending() {
-  return 0;
-}
-
-GWorldPtr fl_create_offscreen(int w, int h) {
-  GWorldPtr gw;
-  Rect bounds;
-  bounds.left=0; bounds.right=w; bounds.top=0; bounds.bottom=h;
-  QDErr err = NewGWorld(&gw, 0, &bounds, 0L, 0L, 0); // 'useTempMem' should not be used (says the Carbon port manual)
-  if ( err == -108 )
-    { }
-//    fl_message( "The application memory is low. Please increase the initial memory assignment.\n" ); 
-  if (err!=noErr || gw==0L) return 0L;
-  return gw;
-}
-
-void fl_copy_offscreen(int x,int y,int w,int h,GWorldPtr gWorld,int srcx,int srcy) {
-  Rect src;
-  if ( !gWorld ) return;
-  src.top = srcy; src.left = srcx; src.bottom = srcy+h; src.right = srcx+w;
-  Rect dst;
-  GrafPtr dstPort; GetPort(&dstPort);
-  dst.top = y; dst.left = x; dst.bottom = y+h; dst.right = x+w;
-  RGBColor rgb, oldbg, oldfg;
-  GetForeColor(&oldfg);
-  GetBackColor(&oldbg);
-  rgb.red = 0xffff; rgb.green = 0xffff; rgb.blue = 0xffff;
-  RGBBackColor( &rgb );
-  rgb.red = 0x0000; rgb.green = 0x0000; rgb.blue = 0x0000;
-  RGBForeColor( &rgb );
-  CopyBits(GetPortBitMapForCopyBits(gWorld), GetPortBitMapForCopyBits(dstPort), &src, &dst, srcCopy, 0L);
-  RGBBackColor(&oldbg);
-  RGBForeColor(&oldfg);
-}
-
-void fl_delete_offscreen(GWorldPtr gWorld) {
-  DisposeGWorld(gWorld);
-}
-
-static GrafPtr prevPort;
-static GDHandle prevGD;
-
-void fl_begin_offscreen(GWorldPtr gWorld) {
-  GetGWorld( &prevPort, &prevGD );
-  if ( gWorld )
-  {
-    SetGWorld( gWorld, 0 ); // sets the correct port
-    PixMapHandle pm = GetGWorldPixMap(gWorld);
-    Boolean ret = LockPixels(pm);
-    if ( ret == false )
-    {
-      Rect rect;
-      GetPortBounds( gWorld, &rect );
-      UpdateGWorld( &gWorld, 0, &rect, 0, 0, 0 );
-      pm = GetGWorldPixMap( gWorld );
-      LockPixels( pm );
-    }
-    fl_window = 0;
-  }
-  fl_push_no_clip();
-}
-
-void fl_end_offscreen() {
-  GWorldPtr currPort;
-  GDHandle currGD;
-  GetGWorld( &currPort, &currGD );
-  fl_pop_clip();
-  PixMapHandle pm = GetGWorldPixMap(currPort);
-  UnlockPixels(pm);
-  SetGWorld( prevPort, prevGD );
-  fl_window = GetWindowFromPort( prevPort );
-}
-
-extern void fl_restore_clip();
-
-#elif defined(__APPLE_QUARTZ__)
+#elif defined(__APPLE_QUARTZ__) || defined(FL_DOXYGEN)
 
 char fl_can_do_alpha_blending() {
   return 1;
 }
 
+#if ! defined(FL_DOXYGEN)
+Fl_Offscreen Fl_Quartz_Graphics_Driver::create_offscreen_with_alpha(int w, int h) {
+  void *data = calloc(w*h,4);
+  CGColorSpaceRef lut = CGColorSpaceCreateDeviceRGB();
+  CGContextRef ctx = CGBitmapContextCreate(
+    data, w, h, 8, w*4, lut, kCGImageAlphaPremultipliedLast);
+  CGColorSpaceRelease(lut);
+  return (Fl_Offscreen)ctx;
+}
+#endif
+
+/** \addtogroup fl_drawings
+ @{
+ */
+
+/** 
+  Creation of an offscreen graphics buffer.
+ \param w,h     width and height in pixels of the buffer.
+ \return    the created graphics buffer.
+ */
 Fl_Offscreen fl_create_offscreen(int w, int h) {
   void *data = calloc(w*h,4);
   CGColorSpaceRef lut = CGColorSpaceCreateDeviceRGB();
@@ -246,23 +266,23 @@ Fl_Offscreen fl_create_offscreen(int w, int h) {
   return (Fl_Offscreen)ctx;
 }
 
-Fl_Offscreen fl_create_offscreen_with_alpha(int w, int h) {
-  void *data = calloc(w*h,4);
-  CGColorSpaceRef lut = CGColorSpaceCreateDeviceRGB();
-  CGContextRef ctx = CGBitmapContextCreate(
-    data, w, h, 8, w*4, lut, kCGImageAlphaPremultipliedLast);
-  CGColorSpaceRelease(lut);
-  return (Fl_Offscreen)ctx;
+static void bmProviderRelease (void *src, const void *data, size_t size) {
+  CFIndex count = CFGetRetainCount(src);
+  CFRelease(src);
+  if(count == 1) free((void*)data);
 }
 
-void fl_copy_offscreen(int x,int y,int w,int h,Fl_Offscreen osrc,int srcx,int srcy) {
+void Fl_Quartz_Graphics_Driver::copy_offscreen(int x,int y,int w,int h,Fl_Offscreen osrc,int srcx,int srcy) {
   CGContextRef src = (CGContextRef)osrc;
   void *data = CGBitmapContextGetData(src);
   int sw = CGBitmapContextGetWidth(src);
   int sh = CGBitmapContextGetHeight(src);
   CGImageAlphaInfo alpha = CGBitmapContextGetAlphaInfo(src);
   CGColorSpaceRef lut = CGColorSpaceCreateDeviceRGB();
-  CGDataProviderRef src_bytes = CGDataProviderCreateWithData( 0L, data, sw*sh*4, 0L);
+  // when output goes to a Quartz printercontext, release of the bitmap must be
+  // delayed after the end of the print page
+  CFRetain(src);
+  CGDataProviderRef src_bytes = CGDataProviderCreateWithData( src, data, sw*sh*4, bmProviderRelease);
   CGImageRef img = CGImageCreate( sw, sh, 8, 4*8, 4*sw, lut, alpha,
     src_bytes, 0L, false, kCGRenderingIntentDefault);
   // fl_push_clip();
@@ -275,20 +295,29 @@ void fl_copy_offscreen(int x,int y,int w,int h,Fl_Offscreen osrc,int srcx,int sr
   CGDataProviderRelease(src_bytes);
 }
 
+/**  Deletion of an offscreen graphics buffer.
+ \param ctx     the buffer to be deleted.
+ */
 void fl_delete_offscreen(Fl_Offscreen ctx) {
   if (!ctx) return;
   void *data = CGBitmapContextGetData((CGContextRef)ctx);
+  CFIndex count = CFGetRetainCount(ctx);
   CGContextRelease((CGContextRef)ctx);
-  if (!data) return;
-  free(data);
+  if(count == 1) free(data);
 }
 
 const int stack_max = 16;
 static int stack_ix = 0;
 static CGContextRef stack_gc[stack_max];
 static Window stack_window[stack_max];
+static Fl_Surface_Device *_ss;
 
+/**  Send all subsequent drawing commands to this offscreen buffer.
+ \param ctx     the offscreen buffer.
+ */
 void fl_begin_offscreen(Fl_Offscreen ctx) {
+  _ss = Fl_Surface_Device::surface(); 
+  Fl_Display_Device::display_device()->set_current();
   if (stack_ix<stack_max) {
     stack_gc[stack_ix] = fl_gc;
     stack_window[stack_ix] = fl_window;
@@ -298,14 +327,15 @@ void fl_begin_offscreen(Fl_Offscreen ctx) {
 
   fl_gc = (CGContextRef)ctx;
   fl_window = 0;
-  //fl_push_no_clip();
   CGContextSaveGState(fl_gc);
-  Fl_X::q_fill_context();
+  fl_push_no_clip();
 }
 
+/** Quit sending drawing commands to the current offscreen buffer.
+ */
 void fl_end_offscreen() {
   Fl_X::q_release_context();
-  //fl_pop_clip();
+  fl_pop_clip();
   if (stack_ix>0)
     stack_ix--;
   else
@@ -314,55 +344,56 @@ void fl_end_offscreen() {
     fl_gc = stack_gc[stack_ix];
     fl_window = stack_window[stack_ix];
   }
+  _ss->set_current();
 }
 
+/** @} */
+
 extern void fl_restore_clip();
 
-#else // X11
-
-// maybe someone feels inclined to implement alpha blending on X11?
-char fl_can_do_alpha_blending() {
-  return 0;
-}
-
+#else
+# error unsupported platform
 #endif
 
-// Fl_Overlay_Window relies on flush(1) copying the back buffer to the
-// front everywhere, even if damage() == 0, thus erasing the overlay,
-// and leaving the clip region set to the entire window.
-
+/**
+  Forces the window to be redrawn.
+*/
 void Fl_Double_Window::flush() {flush(0);}
 
+/**
+  Forces the window to be redrawn.
+  \param[in] eraseoverlay non-zero to erase overlay, zero to ignore
+
+  Fl_Overlay_Window relies on flush(1) copying the back buffer to the
+  front everywhere, even if damage() == 0, thus erasing the overlay,
+  and leaving the clip region set to the entire window.
+*/
 void Fl_Double_Window::flush(int eraseoverlay) {
   make_current(); // make sure fl_gc is non-zero
   Fl_X *myi = Fl_X::i(this);
+  if (!myi) return; // window not yet created
   if (!myi->other_xid) {
 #if USE_XDBE
     if (can_xdbe()) {
-      myi->other_xid =
-        XdbeAllocateBackBufferName(fl_display, fl_xid(this), XdbeCopied);
+      myi->other_xid = XdbeAllocateBackBufferName(fl_display, fl_xid(this), XdbeCopied);
       myi->backbuffer_bad = 1;
     } else
 #endif
-#ifdef __APPLE_QD__
-    if ( ( !QDIsPortBuffered( GetWindowPort(myi->xid) ) ) 
-        || force_doublebuffering_ ) {
-      myi->other_xid = fl_create_offscreen(w(), h());
-      clear_damage(FL_DAMAGE_ALL);
-    }
+#if defined(USE_X11) || defined(WIN32)
+    myi->other_xid = fl_create_offscreen(w(), h());
+    clear_damage(FL_DAMAGE_ALL);
 #elif defined(__APPLE_QUARTZ__)
     if (force_doublebuffering_) {
       myi->other_xid = fl_create_offscreen(w(), h());
       clear_damage(FL_DAMAGE_ALL);
     }
 #else
-    myi->other_xid = fl_create_offscreen(w(), h());
-    clear_damage(FL_DAMAGE_ALL);
+# error unsupported platform
 #endif
   }
 #if USE_XDBE
   if (use_xdbe) {
-    if (myi->backbuffer_bad) {
+    if (myi->backbuffer_bad || eraseoverlay) {
       // Make sure we do a complete redraw...
       if (myi->region) {XDestroyRegion(myi->region); myi->region = 0;}
       clear_damage(FL_DAMAGE_ALL);
@@ -396,6 +427,9 @@ void Fl_Double_Window::flush(int eraseoverlay) {
     RestoreDC(fl_gc, save);
     DeleteDC(fl_gc);
     fl_gc = _sgc;
+    //# if defined(FLTK_USE_CAIRO)
+    //if Fl::cairo_autolink_context() Fl::cairo_make_current(this); // capture gc changes automatically to update the cairo context adequately
+    //# endif
 #elif defined(__APPLE__)
     if ( myi->other_xid ) {
       fl_begin_offscreen( myi->other_xid );
@@ -423,7 +457,15 @@ void Fl_Double_Window::resize(int X,int Y,int W,int H) {
   int oh = h();
   Fl_Window::resize(X,Y,W,H);
 #if USE_XDBE
-  if (use_xdbe) return;
+  if (use_xdbe) {
+    Fl_X* myi = Fl_X::i(this);
+    if (myi && myi->other_xid && (ow < w() || oh < h())) {
+      // STR #2152: Deallocate the back buffer to force creation of a new one.
+      XdbeDeallocateBackBufferName(fl_display,myi->other_xid);
+      myi->other_xid = 0;
+    }
+    return;
+  }
 #endif
   Fl_X* myi = Fl_X::i(this);
   if (myi && myi->other_xid && (ow != w() || oh != h())) {
@@ -443,10 +485,34 @@ void Fl_Double_Window::hide() {
   Fl_Window::hide();
 }
 
+/**
+  The destructor <I>also deletes all the children</I>. This allows a
+  whole tree to be deleted at once, without having to keep a pointer to
+  all the children in the user code.
+*/
 Fl_Double_Window::~Fl_Double_Window() {
   hide();
 }
 
+
+Fl_Overlay_Window::Fl_Overlay_Window(int W, int H, const char *l)
+: Fl_Double_Window(W,H,l) 
+{
+  overlay_ = 0; 
+  force_doublebuffering_=1; 
+  image(0); 
+}
+  
+  
+Fl_Overlay_Window::Fl_Overlay_Window(int X, int Y, int W, int H, const char *l)
+: Fl_Double_Window(X,Y,W,H,l) 
+{
+  overlay_ = 0; 
+  force_doublebuffering_=1; 
+  image(0); 
+}
+
+  
 //
-// End of "$Id: Fl_Double_Window.cxx 5829 2007-05-14 15:51:00Z matt $".
+// End of "$Id: Fl_Double_Window.cxx 9719 2012-11-13 14:45:42Z manolo $".
 //
