@@ -1,24 +1,15 @@
 //
-// "$Id: Fl_File_Browser.cxx 5635 2007-01-23 15:02:00Z mike $"
+// "$Id: Fl_File_Browser.cxx 9325 2012-04-05 05:12:30Z fabien $"
 //
 // Fl_File_Browser routines.
 //
-// Copyright 1999-2006 by Michael Sweet.
+// Copyright 1999-2010 by Michael Sweet.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -42,6 +33,7 @@
 #include <FL/Fl_File_Browser.H>
 #include <FL/fl_draw.H>
 #include <FL/filename.H>
+#include <FL/Fl_Image.H>	// icon
 #include <stdio.h>
 #include <stdlib.h>
 #include "flstring.h"
@@ -64,14 +56,11 @@
 #  include <os2.h>
 #endif // __EMX__
 
-// CodeWarrior (__MWERKS__) gets its include paths confused, so we
-// temporarily disable this...
-#if defined(__APPLE__) && !defined(__MWERKS__)
+#if defined(__APPLE__)
 #  include <sys/param.h>
 #  include <sys/ucred.h>
 #  include <sys/mount.h>
-#endif // __APPLE__ && !__MWERKS__
-
+#endif // __APPLE__
 
 //
 // FL_BLINE definition from "Fl_Browser.cxx"...
@@ -80,11 +69,17 @@
 #define SELECTED 1
 #define NOTDISPLAYED 2
 
+// TODO -- Warning: The definition of FL_BLINE here is a hack.
+//    Fl_File_Browser should not do this. PLEASE FIX.
+//    FL_BLINE should be private to Fl_Browser, and not re-defined here.
+//    For now, make sure this struct is precisely consistent with Fl_Browser.cxx.
+//
 struct FL_BLINE			// data is in a linked list of these
 {
   FL_BLINE	*prev;		// Previous item in list
   FL_BLINE	*next;		// Next item in list
   void		*data;		// Pointer to data (function)
+  Fl_Image      *icon;		// Pointer to optional icon
   short		length;		// sizeof(txt)-1, may be longer than string
   char		flags;		// selected, displayed
   char		txt[1];		// start of allocated array
@@ -497,7 +492,7 @@ Fl_File_Browser::load(const char     *directory,// I - Directory to load
 
 	num_files ++;
       }
-#elif defined(__APPLE__) && !defined(__MWERKS__)
+#elif defined(__APPLE__)
     // MacOS X and Darwin use getfsstat() system call...
     int			numfs;	// Number of file systems
     struct statfs	*fs;	// Buffer for file system info
@@ -532,19 +527,19 @@ Fl_File_Browser::load(const char     *directory,// I - Directory to load
     // UNIX code uses /etc/fstab or similar...
     //
     FILE	*mtab;		// /etc/mtab or /etc/mnttab file
-    char	line[1024];	// Input line
+    char	line[FL_PATH_MAX];	// Input line
 
     //
     // Open the file that contains a list of mounted filesystems...
     //
 
-    mtab = fopen("/etc/mnttab", "r");	// Fairly standard
+    mtab = fl_fopen("/etc/mnttab", "r");	// Fairly standard
     if (mtab == NULL)
-      mtab = fopen("/etc/mtab", "r");	// More standard
+      mtab = fl_fopen("/etc/mtab", "r");	// More standard
     if (mtab == NULL)
-      mtab = fopen("/etc/fstab", "r");	// Otherwise fallback to full list
+      mtab = fl_fopen("/etc/fstab", "r");	// Otherwise fallback to full list
     if (mtab == NULL)
-      mtab = fopen("/etc/vfstab", "r");	// Alternate full list file
+      mtab = fl_fopen("/etc/vfstab", "r");	// Alternate full list file
 
     if (mtab != NULL)
     {
@@ -577,7 +572,7 @@ Fl_File_Browser::load(const char     *directory,// I - Directory to load
 
 #if (defined(WIN32) && !defined(__CYGWIN__)) || defined(__EMX__)
     strlcpy(filename, directory_, sizeof(filename));
-    i = strlen(filename) - 1;
+    i = (int) (strlen(filename) - 1);
 
     if (i == 2 && filename[1] == ':' &&
         (filename[2] == '/' || filename[2] == '\\'))
@@ -635,5 +630,5 @@ Fl_File_Browser::filter(const char *pattern)	// I - Pattern string
 
 
 //
-// End of "$Id: Fl_File_Browser.cxx 5635 2007-01-23 15:02:00Z mike $".
+// End of "$Id: Fl_File_Browser.cxx 9325 2012-04-05 05:12:30Z fabien $".
 //
