@@ -1,24 +1,15 @@
 //
-// "$Id: Fl_Gl_Choice.cxx 6811 2009-06-30 19:05:09Z matt $"
+// "$Id: Fl_Gl_Choice.cxx 8864 2011-07-19 04:49:30Z greg.ercolano $"
 //
 // OpenGL visual selection code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -34,8 +25,10 @@
 #  include "Fl_Gl_Choice.H"
 #  include <FL/gl_draw.H>
 #  include "flstring.h"
+#  include <FL/fl_utf8.h>
 
 #  ifdef __APPLE__
+#    include <ApplicationServices/ApplicationServices.h>
 #    include <FL/Fl_Window.H>
 #  endif
 
@@ -45,7 +38,7 @@ void fl_save_dc(HWND, HDC);
 
 static Fl_Gl_Choice *first;
 
-// this assummes one of the two arguments is zero:
+// this assumes one of the two arguments is zero:
 // We keep the list system in Win32 to stay compatible and interpret
 // the list later...
 Fl_Gl_Choice *Fl_Gl_Choice::find(int m, const int *alistp) {
@@ -55,107 +48,7 @@ Fl_Gl_Choice *Fl_Gl_Choice::find(int m, const int *alistp) {
     if (g->mode == m && g->alist == alistp) 
       return g;
 
-#  ifdef __APPLE_QD__
-  const int *blist;
-  int list[32];
-    
-  if (alistp)
-    blist = alistp;
-  else {
-    int n = 0;
-    if (m & FL_INDEX) {
-      list[n++] = AGL_BUFFER_SIZE;
-      list[n++] = 8; // glut tries many sizes, but this should work...
-    } else {
-      list[n++] = AGL_RGBA;
-      list[n++] = AGL_GREEN_SIZE;
-      list[n++] = (m & FL_RGB8) ? 8 : 1;
-      if (m & FL_ALPHA) {
-	list[n++] = AGL_ALPHA_SIZE;
-	list[n++] = (m & FL_RGB8) ? 8 : 1;
-      }
-      if (m & FL_ACCUM) {
-	list[n++] = AGL_ACCUM_GREEN_SIZE;
-	list[n++] = 1;
-	if (m & FL_ALPHA) {
-	  list[n++] = AGL_ACCUM_ALPHA_SIZE;
-	  list[n++] = 1;
-	}
-      }
-    }
-    if (m & FL_DOUBLE) {
-      list[n++] = AGL_DOUBLEBUFFER;
-    }
-    if (m & FL_DEPTH) {
-      list[n++] = AGL_DEPTH_SIZE; list[n++] = 24;
-    }
-    if (m & FL_STENCIL) {
-      list[n++] = AGL_STENCIL_SIZE; list[n++] = 1;
-    }
-#    ifdef AGL_STEREO
-    if (m & FL_STEREO) {
-      list[n++] = AGL_STEREO;
-    }
-#    endif
-    list[n] = AGL_NONE;
-    blist = list;
-  }
-  fl_open_display();
-  AGLPixelFormat fmt = aglChoosePixelFormat(NULL, 0, (GLint*)blist);
-  if (!fmt) return 0;
-
-#elif defined(__APPLE_QUARTZ__)
-  // warning: the Quartz version should probably use Core GL (CGL) instead of AGL
-  const int *blist;
-  int list[32];
-   
-  if (alistp)
-    blist = alistp;
-  else {
-    int n = 0;
-    if (m & FL_INDEX) {
-      list[n++] = AGL_BUFFER_SIZE;
-      list[n++] = 8; // glut tries many sizes, but this should work...
-    } else {
-      list[n++] = AGL_RGBA;
-      list[n++] = AGL_GREEN_SIZE;
-      list[n++] = (m & FL_RGB8) ? 8 : 1;
-      if (m & FL_ALPHA) {
-        list[n++] = AGL_ALPHA_SIZE;
-        list[n++] = (m & FL_RGB8) ? 8 : 1;
-      }
-      if (m & FL_ACCUM) {
-        list[n++] = AGL_ACCUM_GREEN_SIZE;
-        list[n++] = 1;
-        if (m & FL_ALPHA) {
-          list[n++] = AGL_ACCUM_ALPHA_SIZE;
-          list[n++] = 1;
-        }
-      }
-    }
-    if (m & FL_DOUBLE) {
-      list[n++] = AGL_DOUBLEBUFFER;
-    }
-    if (m & FL_DEPTH) {
-      list[n++] = AGL_DEPTH_SIZE; list[n++] = 24;
-    }
-    if (m & FL_STENCIL) {
-      list[n++] = AGL_STENCIL_SIZE; list[n++] = 1;
-    }
-#    ifdef AGL_STEREO
-    if (m & FL_STEREO) {
-      list[n++] = AGL_STEREO;
-    }
-#    endif
-    list[n] = AGL_NONE;
-    blist = list;
-  }
-  fl_open_display();
-  AGLPixelFormat fmt = aglChoosePixelFormat(NULL, 0, (GLint*)blist);
-  if (!fmt) return 0;
-  
-#  elif !defined(WIN32)    
-
+#if defined(USE_X11)    
   const int *blist;
   int list[32];
     
@@ -214,7 +107,57 @@ Fl_Gl_Choice *Fl_Gl_Choice::find(int m, const int *alistp) {
     return 0;
   }
 
-#  else
+#elif defined(__APPLE_QUARTZ__)
+  // warning: the Quartz version should probably use Core GL (CGL) instead of AGL
+  const int *blist;
+  int list[32];
+   
+  if (alistp)
+    blist = alistp;
+  else {
+    int n = 0;
+    if (m & FL_INDEX) {
+      list[n++] = AGL_BUFFER_SIZE;
+      list[n++] = 8; // glut tries many sizes, but this should work...
+    } else {
+      list[n++] = AGL_RGBA;
+      list[n++] = AGL_GREEN_SIZE;
+      list[n++] = (m & FL_RGB8) ? 8 : 1;
+      if (m & FL_ALPHA) {
+        list[n++] = AGL_ALPHA_SIZE;
+        list[n++] = (m & FL_RGB8) ? 8 : 1;
+      }
+      if (m & FL_ACCUM) {
+        list[n++] = AGL_ACCUM_GREEN_SIZE;
+        list[n++] = 1;
+        if (m & FL_ALPHA) {
+          list[n++] = AGL_ACCUM_ALPHA_SIZE;
+          list[n++] = 1;
+        }
+      }
+    }
+    if (m & FL_DOUBLE) {
+      list[n++] = AGL_DOUBLEBUFFER;
+    }
+    if (m & FL_DEPTH) {
+      list[n++] = AGL_DEPTH_SIZE; list[n++] = 24;
+    }
+    if (m & FL_STENCIL) {
+      list[n++] = AGL_STENCIL_SIZE; list[n++] = 1;
+    }
+#    ifdef AGL_STEREO
+    if (m & FL_STEREO) {
+      list[n++] = AGL_STEREO;
+    }
+#    endif
+    list[n] = AGL_NONE;
+    blist = list;
+  }
+  fl_open_display();
+  AGLPixelFormat fmt = aglChoosePixelFormat(NULL, 0, (GLint*)blist);
+  if (!fmt) return 0;
+  
+#elif defined(WIN32)
 
   // Replacement for ChoosePixelFormat() that finds one with an overlay
   // if possible:
@@ -249,8 +192,9 @@ Fl_Gl_Choice *Fl_Gl_Choice::find(int m, const int *alistp) {
   }
   //printf("Chosen pixel format is %d\n", pixelformat);
   if (!pixelformat) return 0;
-
-#  endif
+#else
+# error platform unsupported
+#endif
 
   g = new Fl_Gl_Choice;
   g->mode = m;
@@ -258,24 +202,24 @@ Fl_Gl_Choice *Fl_Gl_Choice::find(int m, const int *alistp) {
   g->next = first;
   first = g;
 
-#  ifdef WIN32
-  g->pixelformat = pixelformat;
-  g->pfd = chosen_pfd;
-#  elif defined(__APPLE_QD__)
-  g->pixelformat = fmt;
-#  elif defined(__APPLE_QUARTZ__)
-  // warning: the Quartz version should probably use Core GL (CGL) instead of AGL
-  g->pixelformat = fmt;
-#  else
+#if defined(USE_X11)
   g->vis = visp;
 
   if (/*MaxCmapsOfScreen(ScreenOfDisplay(fl_display,fl_screen))==1 && */
       visp->visualid == fl_visual->visualid &&
-      !getenv("MESA_PRIVATE_CMAP"))
+      !fl_getenv("MESA_PRIVATE_CMAP"))
     g->colormap = fl_colormap;
   else
     g->colormap = XCreateColormap(fl_display, RootWindow(fl_display,fl_screen),
 				  visp->visual, AllocNone);
+#  elif defined(WIN32)
+  g->pixelformat = pixelformat;
+  g->pfd = chosen_pfd;
+#  elif defined(__APPLE_QUARTZ__)
+  // warning: the Quartz version should probably use Core GL (CGL) instead of AGL
+  g->pixelformat = fmt;
+#  else
+#    error unsupported platform
 #  endif
 
   return g;
@@ -308,7 +252,18 @@ static void del_context(GLContext ctx) {
   if (!nContext) gl_remove_displaylist_fonts();
 }
 
-#  ifdef WIN32
+#if defined(USE_X11)
+
+GLContext fl_create_gl_context(XVisualInfo* vis) {
+  GLContext shared_ctx = 0;
+  if (context_list && nContext) shared_ctx = context_list[0];
+  GLContext context = glXCreateContext(fl_display, vis, shared_ctx, 1);
+  if (context)
+    add_context(context);
+  return context;
+}
+
+#elif defined(WIN32)
 
 GLContext fl_create_gl_context(Fl_Window* window, const Fl_Gl_Choice* g, int layer) {
   Fl_X* i = Fl_X::i(window);
@@ -331,50 +286,49 @@ GLContext fl_create_gl_context(Fl_Window* window, const Fl_Gl_Choice* g, int lay
   return context;
 }
 
-#  elif defined(__APPLE_QD__)
-GLContext fl_create_gl_context(Fl_Window* window, const Fl_Gl_Choice* g, int layer) {
-    GLContext context, shared_ctx = 0;
-    if (context_list && nContext) shared_ctx = context_list[0];
-    context = aglCreateContext( g->pixelformat, shared_ctx);
-    if (!context) return 0;
-    add_context((GLContext)context);
-    if ( window->parent() ) {
-      Rect wrect; GetWindowPortBounds( fl_xid(window), &wrect );
-      GLint rect[] = { window->x(), wrect.bottom-window->h()-window->y(), window->w(), window->h() }; 
-      aglSetInteger( (GLContext)context, AGL_BUFFER_RECT, rect );
-      aglEnable( (GLContext)context, AGL_BUFFER_RECT );
-    }
-    aglSetDrawable( context, GetWindowPort( fl_xid(window) ) );
-    return (context);
-}
 #  elif defined(__APPLE_QUARTZ__)
-  // warning: the Quartz version should probably use Core GL (CGL) instead of AGL
-  GLContext fl_create_gl_context(Fl_Window* window, const Fl_Gl_Choice* g, int layer) {
-    GLContext context, shared_ctx = 0;
-    if (context_list && nContext) shared_ctx = context_list[0];
-    context = aglCreateContext( g->pixelformat, shared_ctx);
-    if (!context) return 0;
-    add_context((GLContext)context);
-    if ( window->parent() ) {
-      Rect wrect; GetWindowPortBounds( fl_xid(window), &wrect );
-      GLint rect[] = { window->x(), wrect.bottom-window->h()-window->y(), window->w(), window->h() };
-      aglSetInteger( (GLContext)context, AGL_BUFFER_RECT, rect );
-      aglEnable( (GLContext)context, AGL_BUFFER_RECT );
-    }
-    aglSetDrawable( context, GetWindowPort( fl_xid(window) ) );
-    return (context);
+#if !(MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5 && __LP64__)
+static CGrafPtr fl_GetWindowPort(WindowRef window)
+{
+  typedef CGrafPtr (*wf)(WindowRef);
+  static wf f = NULL;
+  if ( ! f) f = (wf)Fl_X::get_carbon_function("GetWindowPort");
+  return (*f)(window);
+}
+#endif
+
+// warning: the Quartz version should probably use Core GL (CGL) instead of AGL
+GLContext fl_create_gl_context(Fl_Window* window, const Fl_Gl_Choice* g, int layer) {
+  GLContext context, shared_ctx = 0;
+  if (context_list && nContext) shared_ctx = context_list[0];
+  context = aglCreateContext( g->pixelformat, shared_ctx);
+  if (!context) return 0;
+  add_context((GLContext)context);
+  if ( window->parent() ) {
+    int H = window->window()->h();
+    GLint rect[] = { window->x(), H-window->h()-window->y(), window->w(), window->h() };
+    aglSetInteger( (GLContext)context, AGL_BUFFER_RECT, rect );
+    aglEnable( (GLContext)context, AGL_BUFFER_RECT );
+  }
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5
+#if __LP64__
+  // 64 bit version
+  aglSetWindowRef(context, Fl_X::i(window)->window_ref() );
+#else
+  // 32 bit version >= 10.5
+  if (aglSetWindowRef != NULL)
+    aglSetWindowRef(context, Fl_X::i(window)->window_ref() );
+  else
+    aglSetDrawable( context, fl_GetWindowPort( Fl_X::i(window)->window_ref() ) );
+#endif
+#else
+  // 32 bit version < 10.5
+  aglSetDrawable( context, fl_GetWindowPort( Fl_X::i(window)->window_ref() ) );
+#endif
+  return (context);
 }
 #  else
-
-GLContext fl_create_gl_context(XVisualInfo* vis) {
-  GLContext shared_ctx = 0;
-  if (context_list && nContext) shared_ctx = context_list[0];
-  GLContext context = glXCreateContext(fl_display, vis, shared_ctx, 1);
-  if (context)
-    add_context(context);
-  return context;
-}
-
+#    error unsupported platform
 #  endif
 
 static GLContext cached_context;
@@ -384,29 +338,36 @@ void fl_set_gl_context(Fl_Window* w, GLContext context) {
   if (context != cached_context || w != cached_window) {
     cached_context = context;
     cached_window = w;
-#  ifdef WIN32
+#  if defined(USE_X11)
+    glXMakeCurrent(fl_display, fl_xid(w), context);
+#  elif defined(WIN32)
     wglMakeCurrent(Fl_X::i(w)->private_dc, context);
-#  elif defined(__APPLE_QD__)
-    if ( w->parent() ) { //: resize our GL buffer rectangle
-      Rect wrect; GetWindowPortBounds( fl_xid(w), &wrect );
-      GLint rect[] = { w->x(), wrect.bottom-w->h()-w->y(), w->w(), w->h() };
-      aglSetInteger( context, AGL_BUFFER_RECT, rect );
-      aglEnable( context, AGL_BUFFER_RECT );
-    }
-    aglSetDrawable(context, GetWindowPort( fl_xid(w) ) ); 
-    aglSetCurrentContext(context);
 #  elif defined(__APPLE_QUARTZ__)
     // warning: the Quartz version should probably use Core GL (CGL) instead of AGL
     if ( w->parent() ) { //: resize our GL buffer rectangle
-      Rect wrect; GetWindowPortBounds( fl_xid(w), &wrect );
-      GLint rect[] = { w->x(), wrect.bottom-w->h()-w->y(), w->w(), w->h() };
+      int H = w->window()->h();
+      GLint rect[] = { w->x(), H-w->h()-w->y(), w->w(), w->h() };
       aglSetInteger( context, AGL_BUFFER_RECT, rect );
       aglEnable( context, AGL_BUFFER_RECT );
     }
-    aglSetDrawable(context, GetWindowPort( fl_xid(w) ) );
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5
+#if __LP64__
+    // 64 bit version
+    aglSetWindowRef(context, Fl_X::i(w)->window_ref() );
+#else
+    // 32 bit version >= 10.5
+    if (aglSetWindowRef != NULL)
+      aglSetWindowRef(context, Fl_X::i(w)->window_ref() );
+    else
+      aglSetDrawable( context, fl_GetWindowPort( Fl_X::i(w)->window_ref() ) );
+#endif
+#else
+    // 32 bit version < 10.5
+    aglSetDrawable( context, fl_GetWindowPort( Fl_X::i(w)->window_ref() ) );
+#endif
     aglSetCurrentContext(context);
 #  else
-    glXMakeCurrent(fl_display, fl_xid(w), context);
+#   error unsupported platform
 #  endif
   }
 }
@@ -414,33 +375,36 @@ void fl_set_gl_context(Fl_Window* w, GLContext context) {
 void fl_no_gl_context() {
   cached_context = 0;
   cached_window = 0;
-#  ifdef WIN32
+#  if defined(USE_X11)
+  glXMakeCurrent(fl_display, 0, 0);
+#  elif defined(WIN32)
   wglMakeCurrent(0, 0);
-#  elif defined(__APPLE_QD__)
-  AGLContext ctx = aglGetCurrentContext();
-  if (ctx) aglSetDrawable(ctx, NULL);    
-  aglSetCurrentContext(0);
 #  elif defined(__APPLE_QUARTZ__)
   // warning: the Quartz version should probably use Core GL (CGL) instead of AGL
-  AGLContext ctx = aglGetCurrentContext();
-  if (ctx) aglSetDrawable(ctx, NULL);    
+  AGLContext ctx = aglGetCurrentContext();  
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5
+  if (aglSetWindowRef != NULL)
+    { if(ctx) aglSetWindowRef(ctx, NULL ); }
+  else
+#endif
+  if(ctx) aglSetDrawable( ctx, NULL );
   aglSetCurrentContext(0);
 #  else
-  glXMakeCurrent(fl_display, 0, 0);
+#    error unsupported platform
 #  endif
 }
 
 void fl_delete_gl_context(GLContext context) {
   if (cached_context == context) fl_no_gl_context();
-#  ifdef WIN32
+#  if defined(USE_X11)
+  glXDestroyContext(fl_display, context);
+#  elif defined(WIN32)
   wglDeleteContext(context);
-#  elif defined(__APPLE_QD__)
-  aglDestroyContext( context );
 #  elif defined(__APPLE_QUARTZ__)
   // warning: the Quartz version should probably use Core GL (CGL) instead of AGL
   aglDestroyContext( context );
 #  else
-  glXDestroyContext(fl_display, context);
+#    error unsupported platform
 #  endif
   del_context(context);
 }
@@ -449,5 +413,5 @@ void fl_delete_gl_context(GLContext context) {
 
 
 //
-// End of "$Id: Fl_Gl_Choice.cxx 6811 2009-06-30 19:05:09Z matt $".
+// End of "$Id: Fl_Gl_Choice.cxx 8864 2011-07-19 04:49:30Z greg.ercolano $".
 //
