@@ -1,24 +1,15 @@
 //
-// "$Id: Fl_Gl_Window.cxx 6807 2009-06-30 13:38:52Z matt $"
+// "$Id: Fl_Gl_Window.cxx 9264 2012-03-05 08:46:30Z manolo $"
 //
 // OpenGL window code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2006 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -28,11 +19,22 @@
 #include "flstring.h"
 #if HAVE_GL
 
+extern int fl_gl_load_plugin;
+#ifdef __APPLE__
+extern void gl_texture_reset();
+#endif
+
+static int temp = fl_gl_load_plugin;
+
 #include <FL/Fl.H>
 #include <FL/x.H>
+#ifdef __APPLE__
+#include <FL/gl.h>
+#endif
 #include "Fl_Gl_Choice.H"
 #include <FL/Fl_Gl_Window.H>
 #include <stdlib.h>
+#include <FL/fl_utf8.h>
 
 ////////////////////////////////////////////////////////////////
 
@@ -59,11 +61,15 @@ static char SWAP_TYPE = 0 ; // 0 = determine it from environment variable
 
 ////////////////////////////////////////////////////////////////
 
+/**  Returns non-zero if the hardware supports the given or current OpenGL  mode. */
 int Fl_Gl_Window::can_do(int a, const int *b) {
   return Fl_Gl_Choice::find(a,b) != 0;
 }
 
 void Fl_Gl_Window::show() {
+#if defined(__APPLE__)
+  int need_redraw = 0;
+#endif
   if (!shown()) {
     if (!g) {
       g = Fl_Gl_Choice::find(mode_,alist);
@@ -81,15 +87,22 @@ void Fl_Gl_Window::show() {
 #if !defined(WIN32) && !defined(__APPLE__)
     Fl_X::make_xid(this, g->vis, g->colormap);
     if (overlay && overlay != this) ((Fl_Gl_Window*)overlay)->show();
+#elif defined(__APPLE__)
+	if( ! parent() ) need_redraw=1;
 #endif
   }
   Fl_Window::show();
 
 #ifdef __APPLE__
   set_visible();
+  if(need_redraw) redraw();//necessary only after creation of a top-level GL window
 #endif /* __APPLE__ */
 }
 
+/**
+  The invalidate() method turns off valid() and is
+  equivalent to calling value(0).
+*/
 void Fl_Gl_Window::invalidate() {
   valid(0);
   context_valid(0);
@@ -101,6 +114,9 @@ void Fl_Gl_Window::invalidate() {
 #endif
 }
 
+/**
+  See const int Fl_Gl_Window::mode() const 
+*/
 int Fl_Gl_Window::mode(int m, const int *a) {
   if (m == mode_ && a == alist) return 0;
 #ifndef __APPLE__
@@ -113,22 +129,23 @@ int Fl_Gl_Window::mode(int m, const int *a) {
   mode_ = m; alist = a;
   if (shown()) {
     g = Fl_Gl_Choice::find(m, a);
-#if defined(WIN32)
-    if (!g || (oldmode^m)&(FL_DOUBLE|FL_STEREO)) {
-      hide();
-      show();
-    }
-#elif defined(__APPLE_QD__)
-    redraw();
-#elif defined(__APPLE_QUARTZ__)
-    // warning: the Quartz version should probably use Core GL (CGL) instead of AGL
-    redraw();
-#else
+
+#if defined(USE_X11)
     // under X, if the visual changes we must make a new X window (yuck!):
     if (!g || g->vis->visualid!=oldg->vis->visualid || (oldmode^m)&FL_DOUBLE) {
       hide();
       show();
     }
+#elif defined(WIN32)
+    if (!g || (oldmode^m)&(FL_DOUBLE|FL_STEREO)) {
+      hide();
+      show();
+    }
+#elif defined(__APPLE_QUARTZ__)
+    // warning: the Quartz version should probably use Core GL (CGL) instead of AGL
+    redraw();
+#else
+#  error unsupported platform
 #endif
   } else {
     g = 0;
@@ -138,6 +155,13 @@ int Fl_Gl_Window::mode(int m, const int *a) {
 
 #define NON_LOCAL_CONTEXT 0x80000000
 
+/**
+  The make_current() method selects the OpenGL context for the
+  widget.  It is called automatically prior to the draw() method
+  being called and can also be used to implement feedback and/or
+  selection within the handle() method.
+*/
+
 void Fl_Gl_Window::make_current() {
 //  puts("Fl_Gl_Window::make_current()");
 //  printf("make_current: context_=%p\n", context_);
@@ -146,6 +170,10 @@ void Fl_Gl_Window::make_current() {
     context_ = fl_create_gl_context(this, g);
     valid(0);
     context_valid(0);
+#ifdef __APPLE__
+    // resets the pile of string textures used to draw strings
+    gl_texture_reset();
+#endif
   }
   fl_set_gl_context(this, context_);
 
@@ -184,6 +212,11 @@ void Fl_Gl_Window::make_current() {
   current_ = this;
 }
 
+/**
+  Sets the projection so 0,0 is in the lower left of the window and each
+  pixel is 1 unit wide/tall.  If you are drawing 2D images, your 
+  draw() method may want to call this if valid() is false.
+*/
 void Fl_Gl_Window::ortho() {
 // Alpha NT seems to have a broken OpenGL that does not like negative coords:
 #ifdef _M_ALPHA
@@ -199,8 +232,14 @@ void Fl_Gl_Window::ortho() {
 #endif
 }
 
+/**
+  The swap_buffers() method swaps the back and front buffers.
+  It is called automatically after the draw() method is called.
+*/
 void Fl_Gl_Window::swap_buffers() {
-#ifdef WIN32
+#if defined(USE_X11)
+  glXSwapBuffers(fl_display, fl_xid(this));
+#elif defined(WIN32)
 #  if HAVE_GL_OVERLAY
   // Do not swap the overlay, to match GLX:
   BOOL ret = wglSwapLayerBuffers(Fl_X::i(this)->private_dc, WGL_SWAP_MAIN_PLANE);
@@ -208,13 +247,17 @@ void Fl_Gl_Window::swap_buffers() {
 #  else
   SwapBuffers(Fl_X::i(this)->private_dc);
 #  endif
-#elif defined(__APPLE_QD__)
-  aglSwapBuffers((AGLContext)context_);
 #elif defined(__APPLE_QUARTZ__)
-  // warning: the Quartz version should probably use Core GL (CGL) instead of AGL
-  aglSwapBuffers((AGLContext)context_);
+  if(overlay != NULL) {
+    //aglSwapBuffers does not work well with overlays under cocoa
+    glReadBuffer(GL_BACK);
+    glDrawBuffer(GL_FRONT);
+    glCopyPixels(0,0,w(),h(),GL_COLOR);
+  }
+  else
+    aglSwapBuffers((AGLContext)context_);
 #else
-  glXSwapBuffers(fl_display, fl_xid(this));
+# error unsupported platform
 #endif
 }
 
@@ -223,29 +266,24 @@ uchar fl_overlay; // changes how fl_color() works
 int fl_overlay_depth = 0;
 #endif
 
+
 void Fl_Gl_Window::flush() {
   uchar save_valid = valid_f_ & 1;
 #if HAVE_GL_OVERLAY && defined(WIN32)
   uchar save_valid_f = valid_f_;
 #endif
 
-#ifdef __APPLE_QD__
-  //: clear previous clipping in this shared port
-  GrafPtr port = GetWindowPort( fl_xid(this) );
-  Rect rect; SetRect( &rect, 0, 0, 0x7fff, 0x7fff );
-  GrafPtr old; GetPort( &old );
-  SetPort( port );
-  ClipRect( &rect );
-  SetPort( old );
-#elif defined(__APPLE_QUARTZ__)
+#if defined(__APPLE_QUARTZ__)
   // warning: the Quartz version should probably use Core GL (CGL) instead of AGL
   //: clear previous clipping in this shared port
-  GrafPtr port = GetWindowPort( fl_xid(this) );
+#if ! __LP64__
+/*GrafPtr port = GetWindowPort( Fl_X::i(this)->window_ref() );
   Rect rect; SetRect( &rect, 0, 0, 0x7fff, 0x7fff );
   GrafPtr old; GetPort( &old );
   SetPort( port );
   ClipRect( &rect );
-  SetPort( old );
+  SetPort( old );*/
+#endif
 #endif
 
 #if HAVE_GL_OVERLAY && defined(WIN32)
@@ -285,19 +323,17 @@ void Fl_Gl_Window::flush() {
     glDrawBuffer(GL_BACK);
 
     if (!SWAP_TYPE) {
-#ifdef __APPLE_QD__
-      SWAP_TYPE = COPY;
-#elif defined __APPLE_QUARTZ__
-      // warning: the Quartz version should probably use Core GL (CGL) instead of AGL
+#if defined (__APPLE_QUARTZ__) || defined (USE_X11)
       SWAP_TYPE = COPY;
 #else
       SWAP_TYPE = UNDEFINED;
 #endif
-      const char* c = getenv("GL_SWAP_TYPE");
+      const char* c = fl_getenv("GL_SWAP_TYPE");
       if (c) {
 	if (!strcmp(c,"COPY")) SWAP_TYPE = COPY;
 	else if (!strcmp(c, "NODAMAGE")) SWAP_TYPE = NODAMAGE;
 	else if (!strcmp(c, "SWAP")) SWAP_TYPE = SWAP;
+	else SWAP_TYPE = UNDEFINED;
       }
     }
 
@@ -312,16 +348,21 @@ void Fl_Gl_Window::flush() {
 
       // don't draw if only the overlay is damaged:
       if (damage() != FL_DAMAGE_OVERLAY || !save_valid) draw();
-      swap_buffers();
+	  swap_buffers();
 
-    } else { // SWAP_TYPE == UNDEFINED
+    } else if (SWAP_TYPE == SWAP){
+      damage(FL_DAMAGE_ALL);
+      draw();
+      if (overlay == this) draw_overlay();
+      swap_buffers();
+    } else if (SWAP_TYPE == UNDEFINED){ // SWAP_TYPE == UNDEFINED
 
       // If we are faking the overlay, use CopyPixels to act like
       // SWAP_TYPE == COPY.  Otherwise overlay redraw is way too slow.
       if (overlay == this) {
 	// don't draw if only the overlay is damaged:
 	if (damage1_ || damage() != FL_DAMAGE_OVERLAY || !save_valid) draw();
-	// we use a seperate context for the copy because rasterpos must be 0
+	// we use a separate context for the copy because rasterpos must be 0
 	// and depth test needs to be off:
 	static GLContext ortho_context = 0;
 	static Fl_Gl_Window* ortho_window = 0;
@@ -350,7 +391,7 @@ void Fl_Gl_Window::flush() {
 
     }
 
-    if (overlay==this) { // fake overlay in front buffer
+    if (overlay==this && SWAP_TYPE != SWAP) { // fake overlay in front buffer
       glDrawBuffer(GL_FRONT);
       draw_overlay();
       glDrawBuffer(GL_BACK);
@@ -389,6 +430,17 @@ void Fl_Gl_Window::resize(int X,int Y,int W,int H) {
   Fl_Window::resize(X,Y,W,H);
 }
 
+/**
+  Returns or sets a pointer to the GLContext that this window is
+  using. This is a system-dependent structure, but it is portable to copy
+  the context from one window to another. You can also set it to NULL,
+  which will force FLTK to recreate the context the next time make_current()
+  is called, this is useful for getting around bugs in OpenGL implementations.
+  
+  If <i>destroy_flag</i> is true the context will be destroyed by
+  fltk when the window is destroyed, or when the mode() is changed, 
+  or the next time context(x) is called.
+*/
 void Fl_Gl_Window::context(void* v, int destroy_flag) {
   if (context_ && !(mode_&NON_LOCAL_CONTEXT)) fl_delete_gl_context(context_);
   context_ = (GLContext)v;
@@ -396,6 +448,9 @@ void Fl_Gl_Window::context(void* v, int destroy_flag) {
   else mode_ |= NON_LOCAL_CONTEXT;
 }    
 
+/**
+  Hides the window and destroys the OpenGL context.
+*/
 void Fl_Gl_Window::hide() {
   context(0);
 #if HAVE_GL_OVERLAY && defined(WIN32)
@@ -407,6 +462,10 @@ void Fl_Gl_Window::hide() {
   Fl_Window::hide();
 }
 
+/**
+  The destructor removes the widget and destroys the OpenGL context
+  associated with it.
+*/
 Fl_Gl_Window::~Fl_Gl_Window() {
   hide();
 //  delete overlay; this is done by ~Fl_Group
@@ -431,10 +490,65 @@ void Fl_Gl_Window::init() {
 #endif // 0
 }
 
+/**
+  You must implement this virtual function if you want to draw into the
+  overlay.  The overlay is cleared before this is called.  You should
+  draw anything that is not clear using OpenGL.  You must use 
+  gl_color(i) to choose colors (it allocates them from the colormap
+  using system-specific calls), and remember that you are in an indexed
+  OpenGL mode and drawing anything other than flat-shaded will probably
+  not work.
+
+  Both this function and Fl_Gl_Window::draw() should check 
+  Fl_Gl_Window::valid() and set the same transformation.  If you
+  don't your code may not work on other systems.  Depending on the OS,
+  and on whether overlays are real or simulated, the OpenGL context may
+  be the same or different between the overlay and main window.
+*/
 void Fl_Gl_Window::draw_overlay() {}
 
 #endif
 
+  /**
+  You \e \b must subclass Fl_Gl_Window and provide an implementation for 
+  draw().  You may also provide an implementation of draw_overlay()
+  if you want to draw into the overlay planes.  You can avoid
+  reinitializing the viewport and lights and other things by checking 
+  valid() at the start of draw() and only doing the
+  initialization if it is false.
+
+  The draw() method can <I>only</I> use OpenGL calls.  Do not
+  attempt to call X, any of the functions in <FL/fl_draw.H>, or glX
+  directly.  Do not call gl_start() or gl_finish().
+
+  If double-buffering is enabled in the window, the back and front
+  buffers are swapped after this function is completed.
+*/
+void Fl_Gl_Window::draw() {
+    Fl::fatal("Fl_Gl_Window::draw() *must* be overriden. Please refer to the documentation.");
+}
+
+
+/**
+ Handle some FLTK events as needed.
+ */
+int Fl_Gl_Window::handle(int event) 
+{
+#ifdef __APPLE_QUARTZ__
+  if (event==FL_HIDE) {
+    // if we are not hidden, just the parent was hidden, so we must throw away the context
+    if (!visible_r())
+      context(0); // remove context without setting the hidden flags
+  }
+  if (event==FL_SHOW) {
+    // if we are not hidden, just the parent was shown, so we must create a new context
+    if (visible_r())
+      show(); //
+  }
+#endif
+  return Fl_Window::handle(event);
+}
+
 //
-// End of "$Id: Fl_Gl_Window.cxx 6807 2009-06-30 13:38:52Z matt $".
+// End of "$Id: Fl_Gl_Window.cxx 9264 2012-03-05 08:46:30Z manolo $".
 //
