@@ -1,24 +1,15 @@
-// "$Id: Fl_Native_File_Chooser_MAC.mm 8784 2011-06-06 12:11:04Z manolo $"
+// "$Id: Fl_Native_File_Chooser_MAC.mm 9685 2012-09-27 12:49:39Z manolo $"
 //
 // FLTK native OS file chooser widget
 //
 // Copyright 1998-2010 by Bill Spitzak and others.
 // Copyright 2004 Greg Ercolano.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems to:
 //
@@ -39,6 +30,7 @@
 
 
 #include <FL/Fl.H>
+#include <FL/x.H>
 #include <FL/Fl_Native_File_Chooser.H>
 #include <FL/Fl_File_Chooser.H>
 #include <FL/filename.H>
@@ -129,8 +121,6 @@ int Fl_Native_File_Chooser::show() {
   // POST BROWSER
   int err = post();
 
-  _filt_total = 0;
-
   return(err);
 }
 
@@ -351,16 +341,14 @@ int Fl_Native_File_Chooser::filters() const {
 
 #import <Cocoa/Cocoa.h>
 #define UNLIKELYPREFIX "___fl_very_unlikely_prefix_"
-#ifndef MAC_OS_X_VERSION_10_6
-#define MAC_OS_X_VERSION_10_6 1060
-#endif
 
 int Fl_Native_File_Chooser::get_saveas_basename(void) {
-  char *q = strdup( [[(NSSavePanel*)_panel filename] fileSystemRepresentation] );
+  char *q = strdup( [[[(NSSavePanel*)_panel URL] path] UTF8String] );
   id delegate = [(NSSavePanel*)_panel delegate];
   if (delegate != nil) {
-    const char *d = [[(NSSavePanel*)_panel directory] fileSystemRepresentation];
+    const char *d = [[[[(NSSavePanel*)_panel URL] path] stringByDeletingLastPathComponent] UTF8String];
     int l = strlen(d) + 1;
+    if (strcmp(d, "/") == 0) l = 1;
     int lu = strlen(UNLIKELYPREFIX);
     // Remove UNLIKELYPREFIX between directory and filename parts
     memmove(q + l, q + l + lu, strlen(q + l + lu) + 1);
@@ -408,7 +396,7 @@ static char *prepareMacFilter(int count, const char *filter, char **patterns) {
     }
     rank++;
     *(q++) = '\n'; 
-    if (*p) p = r + 1;
+    if (*r) p = r + 1; else p = r;
   } while(*p);
   *q = 0;
   return t;
@@ -497,7 +485,34 @@ static NSPopUpButton *createPopupAccessory(NSSavePanel *panel, const char *filte
   [panel setAccessoryView:view];
   return popup;
 }
-  
+
+int Fl_Native_File_Chooser::runmodal()
+{
+  NSString *dir = nil;
+  NSString *fname = nil;
+  NSString *preset = nil;
+  int retval;
+  if (_preset_file) {
+    preset = [[NSString alloc] initWithUTF8String:_preset_file];
+    if (strchr(_preset_file, '/') != NULL) {
+      dir = [[NSString alloc] initWithString:[preset stringByDeletingLastPathComponent]];
+    }
+    fname = [preset lastPathComponent];
+  }
+  if (_directory && !dir) dir = [[NSString alloc] initWithUTF8String:_directory];
+  if (fl_mac_os_version >= 100600) {
+    if (dir) [(NSSavePanel*)_panel setDirectoryURL:[NSURL fileURLWithPath:dir]];
+    if (fname) [(NSSavePanel*)_panel setNameFieldStringValue:fname];
+    retval = [(NSSavePanel*)_panel runModal];
+  }
+  else {
+    retval = [(id)_panel runModalForDirectory:dir file:fname];
+  }
+  [dir release];
+  [preset release];
+  return retval;
+}
+
 // POST BROWSER
 //     Internal use only.
 //     Assumes '_opts' has been initialized.
@@ -545,6 +560,7 @@ int Fl_Native_File_Chooser::post() {
   }
   
   // SHOW THE DIALOG
+  NSWindow *key = [NSApp keyWindow];
   if ( [(NSSavePanel*)_panel isKindOfClass:[NSOpenPanel class]] ) {
     NSPopUpButton *popup = nil;
     if (_filt_total) {
@@ -563,36 +579,21 @@ int Fl_Native_File_Chooser::post() {
       [openDelegate setPopup:popup filter_pattern:_filt_patt];
       [(NSOpenPanel*)_panel setDelegate:openDelegate];
     }
-    NSString *dir = nil;
-    NSString *fname = nil;
-    NSString *preset = nil;
-    if (_preset_file) {
-      preset = [[NSString alloc] initWithUTF8String:_preset_file];
-      if (strchr(_preset_file, '/') != NULL) 
-	dir = [[NSString alloc] initWithString:[preset stringByDeletingLastPathComponent]];
-      fname = [preset lastPathComponent];
-    }
-    if (_directory && !dir) dir = [[NSString alloc] initWithUTF8String:_directory];
-    retval = [(NSOpenPanel*)_panel runModalForDirectory:dir file:fname types:nil];	
-    [dir release];
-    [preset release];
+    retval = runmodal();
     if (_filt_total) {
       _filt_value = [popup indexOfSelectedItem];
     }
     if ( retval == NSOKButton ) {
       clear_pathnames();
-      NSArray *array = [(NSOpenPanel*)_panel filenames];
+      NSArray *array = [(NSOpenPanel*)_panel URLs];
       _tpathnames = [array count];
       _pathnames = new char*[_tpathnames];
       for(int i = 0; i < _tpathnames; i++) {
-	_pathnames[i] = strnew([(NSString*)[array objectAtIndex:i] fileSystemRepresentation]);
+	_pathnames[i] = strnew([[(NSURL*)[array objectAtIndex:i] path] UTF8String]);
       }
     }
   }
   else {
-    NSString *dir = nil;
-    NSString *fname = nil;
-    NSString *preset = nil;
     NSPopUpButton *popup = nil;
     [(NSSavePanel*)_panel setAllowsOtherFileTypes:YES];
     if ( !(_options & SAVEAS_CONFIRM) ) {
@@ -600,27 +601,19 @@ int Fl_Native_File_Chooser::post() {
       if (saveDelegate == nil)saveDelegate = [[FLsaveDelegate alloc] init]; // not to be ever freed
       [(NSSavePanel*)_panel setDelegate:saveDelegate];
     }
-    if (_preset_file) {
-      preset = [[NSString alloc] initWithUTF8String:_preset_file];
-      if (strchr(_preset_file, '/') != NULL) {
-	dir = [[NSString alloc] initWithString:[preset stringByDeletingLastPathComponent]];
-      }
-      fname = [preset lastPathComponent];
-    }
-    if (_directory && !dir) dir = [[NSString alloc] initWithUTF8String:_directory];
     if (_filt_total) {
+      if (_filt_value >= _filt_total) _filt_value = _filt_total - 1;
       char *t = prepareMacFilter(_filt_total, _filter, _filt_patt);
       popup = createPopupAccessory((NSSavePanel*)_panel, t, [[(NSSavePanel*)_panel nameFieldLabel] UTF8String], _filt_value);
       delete[] t;
     }
-    retval = [(NSSavePanel*)_panel runModalForDirectory:dir file:fname];
+    retval = runmodal();
     if (_filt_total) {
       _filt_value = [popup indexOfSelectedItem];
     }
-    [dir release];
-    [preset release];
     if ( retval == NSOKButton ) get_saveas_basename();
   }
+  [key makeKeyWindow];
   [localPool release];
   return (retval == NSOKButton ? 0 : 1);
 }
@@ -628,5 +621,5 @@ int Fl_Native_File_Chooser::post() {
 #endif // __APPLE__
 
 //
-// End of "$Id: Fl_Native_File_Chooser_MAC.mm 8784 2011-06-06 12:11:04Z manolo $".
+// End of "$Id: Fl_Native_File_Chooser_MAC.mm 9685 2012-09-27 12:49:39Z manolo $".
 //
