@@ -1,25 +1,20 @@
 //
-// "$Id: Fl_Table.cxx 7950 2010-12-05 01:22:53Z greg.ercolano $"
+// "$Id: Fl_Table.cxx 9706 2012-11-06 20:46:14Z matt $"
 //
 // Fl_Table -- A table widget
 //
 // Copyright 2002 by Greg Ercolano.
 // Copyright (c) 2004 O'ksi'D
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-// 
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
+//
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 #include <stdio.h>		// fprintf
@@ -30,8 +25,6 @@
 #include <FL/fl_utf8.H>	// currently only Windows and Linux
 #endif
 
-#define SCROLLBAR_SIZE	16
-
 // Scroll display so 'row' is at top
 void Fl_Table::row_position(int row) {
   if ( _row_position == row ) return;		// OPTIMIZATION: no change? avoid redraw
@@ -131,20 +124,22 @@ Fl_Table::Fl_Table(int X, int Y, int W, int H, const char *l) : Fl_Group(X,Y,W,H
   _dragging_y       = -1;
   _last_row         = -1;
   _auto_drag        = 0;
-  current_col	      = -1;
+  current_col	    = -1;
   current_row       = -1;
   select_row        = -1;
   select_col        = -1;
-  
+#if FLTK_ABI_VERSION >= 10301
+  _scrollbar_size   = 0;
+#endif  
   box(FL_THIN_DOWN_FRAME);
   
-  vscrollbar = new Fl_Scrollbar(x()+w()-SCROLLBAR_SIZE, y(),
-                                SCROLLBAR_SIZE, h()-SCROLLBAR_SIZE);
+  vscrollbar = new Fl_Scrollbar(x()+w()-Fl::scrollbar_size(), y(),
+                                Fl::scrollbar_size(), h()-Fl::scrollbar_size());
   vscrollbar->type(FL_VERTICAL);
   vscrollbar->callback(scroll_cb, (void*)this);
   
-  hscrollbar = new Fl_Scrollbar(x(), y()+h()-SCROLLBAR_SIZE,
-                                w(), SCROLLBAR_SIZE);
+  hscrollbar = new Fl_Scrollbar(x(), y()+h()-Fl::scrollbar_size(),
+                                w(), Fl::scrollbar_size());
   hscrollbar->type(FL_HORIZONTAL);
   hscrollbar->callback(scroll_cb, (void*)this);
   
@@ -485,14 +480,21 @@ void Fl_Table::recalc_dimensions() {
     // First pass: can hide via window size?
     int hidev = (table_h <= tih);
     int hideh = (table_w <= tiw); 
+#if FLTK_ABI_VERSION >= 10301
+    // NEW
+    int scrollsize = _scrollbar_size ? _scrollbar_size : Fl::scrollbar_size();
+#else
+    // OLD
+    int scrollsize = Fl::scrollbar_size();
+#endif
     // Second pass: Check for interference
-    if ( !hideh & hidev ) { hidev = (( table_h - tih + SCROLLBAR_SIZE ) <= 0 ); } 
-    if ( !hidev & hideh ) { hideh = (( table_w - tiw + SCROLLBAR_SIZE ) <= 0 ); } 
+    if ( !hideh & hidev ) { hidev = (( table_h - tih + scrollsize ) <= 0 ); } 
+    if ( !hidev & hideh ) { hideh = (( table_w - tiw + scrollsize ) <= 0 ); } 
     // Determine scrollbar visibility, trim ti[xywh]/to[xywh]
     if ( hidev ) { vscrollbar->hide(); } 
-    else { vscrollbar->show(); tiw -= SCROLLBAR_SIZE; tow -= SCROLLBAR_SIZE; }
+    else { vscrollbar->show(); tiw -= scrollsize; tow -= scrollsize; }
     if ( hideh ) { hscrollbar->hide(); } 
-    else { hscrollbar->show(); tih -= SCROLLBAR_SIZE; toh -= SCROLLBAR_SIZE; }
+    else { hscrollbar->show(); tih -= scrollsize; toh -= scrollsize; }
   } 
   // Resize the child table
   table->resize(tox, toy, tow, toh);
@@ -559,20 +561,27 @@ void Fl_Table::table_resized() {
     // Vertical scrollbar
     float vscrolltab = ( table_h == 0 || tih > table_h ) ? 1 : (float)tih / table_h;
     float hscrolltab = ( table_w == 0 || tiw > table_w ) ? 1 : (float)tiw / table_w;
+#if FLTK_ABI_VERSION >= 10301
+    // NEW
+    int scrollsize = _scrollbar_size ? _scrollbar_size : Fl::scrollbar_size();
+#else
+    // OLD
+    int scrollsize = Fl::scrollbar_size();
+#endif
     vscrollbar->bounds(0, table_h-tih);
     vscrollbar->precision(10);
     vscrollbar->slider_size(vscrolltab);
-    vscrollbar->resize(wix+wiw-SCROLLBAR_SIZE, wiy,
-                       SCROLLBAR_SIZE, 
-                       wih - ((hscrollbar->visible())?SCROLLBAR_SIZE:0));
+    vscrollbar->resize(wix+wiw-scrollsize, wiy,
+                       scrollsize, 
+                       wih - ((hscrollbar->visible())?scrollsize:0));
     vscrollbar->Fl_Valuator::value(vscrollbar->clamp(vscrollbar->value()));	
     // Horizontal scrollbar
     hscrollbar->bounds(0, table_w-tiw);
     hscrollbar->precision(10);
     hscrollbar->slider_size(hscrolltab);
-    hscrollbar->resize(wix, wiy+wih-SCROLLBAR_SIZE,
-                       wiw - ((vscrollbar->visible())?SCROLLBAR_SIZE:0), 
-                       SCROLLBAR_SIZE);
+    hscrollbar->resize(wix, wiy+wih-scrollsize,
+                       wiw - ((vscrollbar->visible())?scrollsize:0), 
+                       scrollsize);
     hscrollbar->Fl_Valuator::value(hscrollbar->clamp(hscrollbar->value()));
   }
   
@@ -689,9 +698,9 @@ int Fl_Table::move_cursor(int R, int C) {
 
 // #define DEBUG 1
 #ifdef DEBUG
-#include "eventnames.h"
+#include <FL/names.h>
 #define PRINTEVENT \
-fprintf(stderr,"Table %s: ** Event: %s --\n", (label()?label():"none"), eventnames[event]);
+    fprintf(stderr,"Table %s: ** Event: %s --\n", (label()?label():"none"), fl_eventnames[event]);
 #else
 #define PRINTEVENT
 #endif
@@ -700,18 +709,33 @@ fprintf(stderr,"Table %s: ** Event: %s --\n", (label()?label():"none"), eventnam
 int Fl_Table::handle(int event) {
   PRINTEVENT;
   int ret = Fl_Group::handle(event);	// let FLTK group handle events first
-  if (ret) {
-    if (Fl::event_inside(hscrollbar) || Fl::event_inside(vscrollbar)) return 1;
-    if (Fl::focus() != this && contains(Fl::focus())) return 1;
-  }
   // Which row/column are we over?
   int R, C;  				// row/column being worked on
   ResizeFlag resizeflag;		// which resizing area are we over? (0=none)
   TableContext context = cursor2rowcol(R, C, resizeflag);
+  if (ret) {
+    if (Fl::event_inside(hscrollbar) || Fl::event_inside(vscrollbar)) return 1;
+    if ( context != CONTEXT_ROW_HEADER &&		// mouse not in row header (STR#2742)
+         context != CONTEXT_COL_HEADER &&		// mouse not in col header (STR#2742)
+         Fl::focus() != this && 			// we don't have focus?
+         contains(Fl::focus())) {			// focus is a child?
+      return 1;
+    }
+  }
+  // Make snapshots of realtime event states *before* we service user's cb,
+  // which may do things like post popup menus that return with unexpected button states.
+  int _event_button = Fl::event_button();
+  int _event_clicks = Fl::event_clicks();
+  int _event_x      = Fl::event_x();
+  int _event_y      = Fl::event_y();
+  int _event_key    = Fl::event_key();
+  int _event_state  = Fl::event_state();
+  Fl_Widget *_focus = Fl::focus();
   switch ( event ) {
     case FL_PUSH:
-      if (Fl::event_button() == 1 && !Fl::event_clicks()) {
-        if (Fl::focus() != this) {
+      // Single left-click on table? do user's callback with CONTEXT_TABLE
+      if (_event_button == 1 && !_event_clicks) {
+        if (_focus == this) {
           take_focus();
           do_callback(CONTEXT_TABLE, -1, -1);
           ret = 1;
@@ -726,21 +750,27 @@ int Fl_Table::handle(int event) {
           current_col = select_col = -1;
         }
       }
-      // Need this for eg. right click to pop up a menu
+      // A click on table with user's callback defined?
+      //     Need this for eg. right click to pop up a menu
+      //
       if ( Fl_Widget::callback() &&		// callback defined?
-          resizeflag == RESIZE_NONE ) {	// not resizing?
-        do_callback(context, R, C);		// do callback
+          resizeflag == RESIZE_NONE ) {		// not resizing?
+        do_callback(context, R, C);		// do callback with context (cell, header, etc)
       }
+      // Handle selection if handling a left-click
+      //    Use snapshot of _event_button we made before servicing user's cb's
+      //    to avoid checking realtime state of buttons which may have changed
+      //    during the user's callbacks.
+      //
       switch ( context ) {
         case CONTEXT_CELL:
           // FL_PUSH on a cell?
-          ret = 1; 			// express interest in FL_RELEASE
+          ret = 1;				// express interest in FL_RELEASE
           break;
           
         case CONTEXT_NONE:
           // FL_PUSH on table corner?
-          if ( Fl::event_button() == 1 && 
-              Fl::event_x() < x() + row_header_width()) {
+          if ( _event_button == 1 && _event_x < x() + row_header_width()) {
             current_col = 0;
             select_col = cols() - 1;
             current_row = 0;
@@ -752,7 +782,7 @@ int Fl_Table::handle(int event) {
           
         case CONTEXT_COL_HEADER:
           // FL_PUSH on a column header?
-          if ( Fl::event_button() == 1) {
+          if ( _event_button == 1) {
             // Resizing? Handle it
             if ( resizeflag ) {
               // Start resize if left click on column border.
@@ -762,7 +792,7 @@ int Fl_Table::handle(int event) {
               //
               _resizing_col = ( resizeflag & RESIZE_COL_LEFT ) ? C-1 : C; 
               _resizing_row = -1;
-              _dragging_x = Fl::event_x(); 
+              _dragging_x = _event_x;
               ret = 1;
             } else {
               // Not resizing? Select the column
@@ -778,7 +808,7 @@ int Fl_Table::handle(int event) {
           
         case CONTEXT_ROW_HEADER:
           // FL_PUSH on a row header?
-          if ( Fl::event_button() == 1 ) {
+          if ( _event_button == 1 ) {
             // Resizing? Handle it
             if ( resizeflag ) {
               // Start resize if left mouse clicked on row border.
@@ -788,7 +818,7 @@ int Fl_Table::handle(int event) {
               //
               _resizing_row = ( resizeflag & RESIZE_ROW_ABOVE ) ? R-1 : R; 
               _resizing_col = -1;
-              _dragging_y = Fl::event_y(); 
+              _dragging_y = _event_y; 
               ret = 1;
             } else {
               // Not resizing? Select the row
@@ -821,11 +851,11 @@ int Fl_Table::handle(int event) {
         //    Don't allow column width smaller than 1.
         //    Continue to show FL_CURSOR_WE at all times during drag.
         //
-        int offset = _dragging_x - Fl::event_x();
+        int offset = _dragging_x - _event_x;
         int new_w = col_width(_resizing_col) - offset;
         if ( new_w < _col_resize_min ) new_w = _col_resize_min;
         col_width(_resizing_col, new_w);
-        _dragging_x = Fl::event_x();
+        _dragging_x = _event_x;
         table_resized();
         redraw();
         change_cursor(FL_CURSOR_WE);
@@ -841,11 +871,11 @@ int Fl_Table::handle(int event) {
         //    Don't allow row width smaller than 1.
         //    Continue to show FL_CURSOR_NS at all times during drag.
         //
-        int offset = _dragging_y - Fl::event_y();
+        int offset = _dragging_y - _event_y;
         int new_h = row_height(_resizing_row) - offset;
         if ( new_h < _row_resize_min ) new_h = _row_resize_min;
         row_height(_resizing_row, new_h);
-        _dragging_y = Fl::event_y();
+        _dragging_y = _event_y;
         table_resized();
         redraw();
         change_cursor(FL_CURSOR_NS);
@@ -854,7 +884,7 @@ int Fl_Table::handle(int event) {
           do_callback(CONTEXT_RC_RESIZE, R, C);
         }
       } else {
-        if (Fl::event_button() == 1 && 
+        if (_event_button == 1 && 
             _selecting == CONTEXT_CELL &&
             context == CONTEXT_CELL) {
           if (select_row != R || select_col != C) {
@@ -864,7 +894,7 @@ int Fl_Table::handle(int event) {
           select_col = C;
           ret = 1;
         }
-        else if (Fl::event_button() == 1 && 
+        else if (_event_button == 1 && 
                  _selecting == CONTEXT_ROW_HEADER && 
                  context & (CONTEXT_ROW_HEADER|CONTEXT_COL_HEADER|CONTEXT_CELL)) {
           if (select_row != R) {
@@ -873,7 +903,7 @@ int Fl_Table::handle(int event) {
           select_row = R;
           ret = 1;
         }
-        else if (Fl::event_button() == 1 && 
+        else if (_event_button == 1 && 
                  _selecting == CONTEXT_COL_HEADER 
                  && context & (CONTEXT_ROW_HEADER|CONTEXT_COL_HEADER|CONTEXT_CELL)) {
           if (select_col != C) {
@@ -885,10 +915,10 @@ int Fl_Table::handle(int event) {
       }
       // Enable autodrag if not resizing, and mouse has moved off table edge
       if ( _resizing_row < 0 && _resizing_col < 0 && _auto_drag == 0 && 
-          ( Fl::event_x() > x() + w() - 20 ||
-           Fl::event_x() < x() + row_header_width() || 
-           Fl::event_y() > y() + h() - 20 ||
-           Fl::event_y() < y() + col_header_height()
+          ( _event_x > x() + w() - 20 ||
+            _event_x < x() + row_header_width() || 
+            _event_y > y() + h() - 20 ||
+            _event_y < y() + col_header_height()
            ) ) {
             _start_auto_drag();
           }
@@ -903,7 +933,7 @@ int Fl_Table::handle(int event) {
         case CONTEXT_TABLE:			// release on dead zone
           if ( _resizing_col == -1 &&		// not resizing a column
               _resizing_row == -1 &&		// not resizing a row
-              Fl_Widget::callback() && 	// callback defined
+              Fl_Widget::callback() && 		// callback defined
               when() & FL_WHEN_RELEASE && 	// on button release
               _last_row == R ) {		// release on same row PUSHed?
             // Need this for eg. left clicking on a cell to select it
@@ -914,7 +944,7 @@ int Fl_Table::handle(int event) {
         default:
           break;
       }
-      if ( Fl::event_button() == 1 ) {
+      if ( _event_button == 1 ) {
         change_cursor(FL_CURSOR_DEFAULT);
         _resizing_col = -1;
         _resizing_row = -1;
@@ -964,7 +994,7 @@ int Fl_Table::handle(int event) {
       ret = 0;
       int is_row = select_row;
       int is_col = select_col;
-      switch(Fl::event_key()) {
+      switch(_event_key) {
         case FL_Home:
           ret = move_cursor(0, -1000000);
           break;
@@ -990,7 +1020,7 @@ int Fl_Table::handle(int event) {
           ret = move_cursor(1, 0);
           break;
 	case FL_Tab:
-	  if ( Fl::event_state() & FL_SHIFT ) {
+	  if ( _event_state & FL_SHIFT ) {
             ret = move_cursor(0, -1);		// shift-tab -> left
 	  } else {
 	    ret = move_cursor(0, 1);		// tab -> right
@@ -1117,6 +1147,20 @@ void Fl_Table::set_selection(int row_top, int col_left, int row_bot, int col_rig
 //    Then tell the group to draw over us.
 //
 void Fl_Table::draw() {   
+#if FLTK_ABI_VERSION >= 10301
+    // NEW
+    int scrollsize = _scrollbar_size ? _scrollbar_size : Fl::scrollbar_size();
+#else
+    // OLD
+    int scrollsize = Fl::scrollbar_size();
+#endif
+  // Check if scrollbar size changed
+  if ( ( vscrollbar && (scrollsize != vscrollbar->w()) ) || 
+       ( hscrollbar && (scrollsize != hscrollbar->h()) ) ) {
+    // handle size change, min/max, table dim's, etc
+    table_resized();
+  }
+
   draw_cell(CONTEXT_STARTPAGE, 0, 0,	 	// let user's drawing routine
             tix, tiy, tiw, tih);		// prep new page
   
@@ -1230,7 +1274,7 @@ void Fl_Table::draw() {
           //
           fl_rectf(wix, tiy + table_h, row_header_width(), 
                    (wiy+wih) - (tiy+table_h) - 
-                   ( hscrollbar->visible() ? SCROLLBAR_SIZE : 0),
+                   ( hscrollbar->visible() ? scrollsize : 0),
                    color());
         }
       }
@@ -1249,5 +1293,5 @@ void Fl_Table::draw() {
 }
 
 //
-// End of "$Id: Fl_Table.cxx 7950 2010-12-05 01:22:53Z greg.ercolano $".
+// End of "$Id: Fl_Table.cxx 9706 2012-11-06 20:46:14Z matt $".
 //
