@@ -1,24 +1,15 @@
 //
-// "$Id: Fl_XPM_Image.cxx 5190 2006-06-09 16:16:34Z mike $"
+// "$Id: Fl_XPM_Image.cxx 9325 2012-04-05 05:12:30Z fabien $"
 //
 // Fl_XPM_Image routines.
 //
-// Copyright 1997-2005 by Bill Spitzak and others.
+// Copyright 1997-2010 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -36,6 +27,7 @@
 #include <FL/Fl_XPM_Image.H>
 #include <stdio.h>
 #include <stdlib.h>
+#include <FL/fl_utf8.h>
 #include "flstring.h"
 
 
@@ -52,11 +44,15 @@ static int hexdigit(int x) {	// I - Hex digit...
 
 #define MAXSIZE 2048
 #define INITIALLINES 256
-
+/**
+  The constructor loads the XPM image from the name filename.
+  <P>The destructor free all memory and server resources that are used by
+  the image.
+*/
 Fl_XPM_Image::Fl_XPM_Image(const char *name) : Fl_Pixmap((char *const*)0) {
   FILE *f;
 
-  if ((f = fopen(name, "rb")) == NULL) return;
+  if ((f = fl_fopen(name, "rb")) == NULL) return;
 
   // read all the c-strings out of the file:
   char** new_data = new char *[INITIALLINES];
@@ -72,7 +68,7 @@ Fl_XPM_Image::Fl_XPM_Image(const char *name) : Fl_Pixmap((char *const*)0) {
       if (*q == '\\') switch (*++q) {
       case '\r':
       case '\n':
-	fgets(q,(buffer+MAXSIZE+20)-q,f); break;
+	if (!fgets(q,(int) (buffer+MAXSIZE+20-q),f)) { /* no problem if we hit EOF */ } break;
       case 0:
 	break;
       case 'x': {
@@ -127,5 +123,5 @@ Fl_XPM_Image::Fl_XPM_Image(const char *name) : Fl_Pixmap((char *const*)0) {
 
 
 //
-// End of "$Id: Fl_XPM_Image.cxx 5190 2006-06-09 16:16:34Z mike $".
+// End of "$Id: Fl_XPM_Image.cxx 9325 2012-04-05 05:12:30Z fabien $".
 //
