@@ -1,24 +1,15 @@
 //
-// "$Id: Fl_x.cxx 8764 2011-05-30 16:47:48Z manolo $"
+// "$Id: Fl_x.cxx 9699 2012-10-16 15:35:34Z manolo $"
 //
 // X specific code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2011 by Bill Spitzak and others.
+// Copyright 1998-2012 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -52,11 +43,18 @@
 #  include <X11/Xlocale.h>
 #  include <X11/Xlib.h>
 #  include <X11/keysym.h>
+#define USE_XRANDR (HAVE_DLSYM && HAVE_DLFCN_H) // means attempt to dynamically load libXrandr.so
+#if USE_XRANDR
+#include <dlfcn.h>
+#define RRScreenChangeNotifyMask  (1L << 0) // from X11/extensions/Xrandr.h
+#define RRScreenChangeNotify	0           // from X11/extensions/Xrandr.h
+typedef int (*XRRUpdateConfiguration_type)(XEvent *event);
+static XRRUpdateConfiguration_type XRRUpdateConfiguration_f;
+static int randrEventBase;                  // base of RandR-defined events
+#endif
 
 static Fl_Xlib_Graphics_Driver fl_xlib_driver;
 static Fl_Display_Device fl_xlib_display(&fl_xlib_driver);
-FL_EXPORT Fl_Graphics_Driver *fl_graphics_driver = (Fl_Graphics_Driver*)&fl_xlib_driver; // the current target device of graphics operations
-Fl_Surface_Device* Fl_Surface_Device::_surface = (Fl_Surface_Device*)&fl_xlib_display; // the current target surface of graphics operations
 Fl_Display_Device *Fl_Display_Device::_display = &fl_xlib_display;// the platform display
 
 ////////////////////////////////////////////////////////////////
@@ -328,6 +326,10 @@ Atom fl_XaUtf8String;
 Atom fl_XaTextUriList;
 Atom fl_NET_WM_NAME;			// utf8 aware window label
 Atom fl_NET_WM_ICON_NAME;		// utf8 aware window icon name
+Atom fl_NET_SUPPORTING_WM_CHECK;
+Atom fl_NET_WM_STATE;
+Atom fl_NET_WM_STATE_FULLSCREEN;
+Atom fl_NET_WORKAREA;
 
 /*
   X defines 32-bit-entities to have a format value of max. 32,
@@ -596,6 +598,7 @@ void fl_open_display() {
   fl_open_display(d);
 }
 
+
 void fl_open_display(Display* d) {
   fl_display = d;
 
@@ -625,6 +628,10 @@ void fl_open_display(Display* d) {
   fl_XaTextUriList      = XInternAtom(d, "text/uri-list",       0);
   fl_NET_WM_NAME        = XInternAtom(d, "_NET_WM_NAME",        0);
   fl_NET_WM_ICON_NAME   = XInternAtom(d, "_NET_WM_ICON_NAME",   0);
+  fl_NET_SUPPORTING_WM_CHECK = XInternAtom(d, "_NET_SUPPORTING_WM_CHECK", 0);
+  fl_NET_WM_STATE       = XInternAtom(d, "_NET_WM_STATE",       0);
+  fl_NET_WM_STATE_FULLSCREEN = XInternAtom(d, "_NET_WM_STATE_FULLSCREEN", 0);
+  fl_NET_WORKAREA       = XInternAtom(d, "_NET_WORKAREA",       0);
 
   if (sizeof(Atom) < 4)
     atom_bits = sizeof(Atom) * 8;
@@ -646,6 +653,24 @@ void fl_open_display(Display* d) {
 #if !USE_COLORMAP
   Fl::visual(FL_RGB);
 #endif
+#if USE_XRANDR
+  void *libxrandr_addr = dlopen("libXrandr.so.2", RTLD_LAZY);
+  if (!libxrandr_addr)  libxrandr_addr = dlopen("libXrandr.so", RTLD_LAZY);
+  if (libxrandr_addr) {
+    int error_base;
+    typedef Bool (*XRRQueryExtension_type)(Display*, int*, int*);
+    typedef void (*XRRSelectInput_type)(Display*, Window, int);
+    XRRQueryExtension_type XRRQueryExtension_f = (XRRQueryExtension_type)dlsym(libxrandr_addr, "XRRQueryExtension");
+    XRRSelectInput_type XRRSelectInput_f = (XRRSelectInput_type)dlsym(libxrandr_addr, "XRRSelectInput");
+    XRRUpdateConfiguration_f = (XRRUpdateConfiguration_type)dlsym(libxrandr_addr, "XRRUpdateConfiguration");
+    if (XRRQueryExtension_f && XRRSelectInput_f && XRRQueryExtension_f(d, &randrEventBase, &error_base))
+      XRRSelectInput_f(d, RootWindow(d, fl_screen), RRScreenChangeNotifyMask);
+    else XRRUpdateConfiguration_f = NULL;
+    }
+#endif
+
+  // Listen for changes to _NET_WORKAREA
+  XSelectInput(d, RootWindow(d, fl_screen), PropertyChangeMask);
 }
 
 void fl_close_display() {
@@ -658,22 +683,26 @@ static int fl_workarea_xywh[4] = { -1, -1, -1, -1 };
 static void fl_init_workarea() {
   fl_open_display();
 
-  Atom _NET_WORKAREA = XInternAtom(fl_display, "_NET_WORKAREA", 0);
   Atom actual;
   unsigned long count, remaining;
   int format;
   unsigned *xywh;
 
-  if (XGetWindowProperty(fl_display, RootWindow(fl_display, fl_screen),
-                         _NET_WORKAREA, 0, 4 * sizeof(unsigned), False,
+  /* If there are several screens, the _NET_WORKAREA property 
+   does not give the work area of the main screen, but that of all screens together.
+   Therefore, we use this property only when there is a single screen,
+   and fall back to the main screen full area when there are several screens.
+   */
+  if (Fl::screen_count() > 1 || XGetWindowProperty(fl_display, RootWindow(fl_display, fl_screen),
+			 fl_NET_WORKAREA, 0, 4 * sizeof(unsigned), False,
                          XA_CARDINAL, &actual, &format, &count, &remaining,
                          (unsigned char **)&xywh) || !xywh || !xywh[2] ||
                          !xywh[3])
   {
-    fl_workarea_xywh[0] = 0;
-    fl_workarea_xywh[1] = 0;
-    fl_workarea_xywh[2] = DisplayWidth(fl_display, fl_screen);
-    fl_workarea_xywh[3] = DisplayHeight(fl_display, fl_screen);
+    Fl::screen_xywh(fl_workarea_xywh[0], 
+		    fl_workarea_xywh[1], 
+		    fl_workarea_xywh[2], 
+		    fl_workarea_xywh[3], 0);
   }
   else
   {
@@ -768,6 +797,31 @@ void fl_sendClientMessage(Window window, Atom message,
   XSendEvent(fl_display, window, 0, 0, &e);
 }
 
+
+/* 
+   Get window property value (32 bit format) 
+   Returns zero on success, -1 on error
+*/
+static int get_xwinprop(Window wnd, Atom prop, long max_length,
+                        unsigned long *nitems, unsigned long **data) {
+  Atom actual;
+  int format;
+  unsigned long bytes_after;
+  
+  if (Success != XGetWindowProperty(fl_display, wnd, prop, 0, max_length, 
+                                    False, AnyPropertyType, &actual, &format, 
+                                    nitems, &bytes_after, (unsigned char**)data)) {
+    return -1;
+  }
+
+  if (actual == None || format != 32) {
+    return -1;
+  }
+
+  return 0;
+}
+
+
 ////////////////////////////////////////////////////////////////
 // Code for copying to clipboard and DnD out of the program:
 
@@ -924,7 +978,20 @@ int fl_handle(const XEvent& thisevent)
 
   if ( XFilterEvent((XEvent *)&xevent, 0) )
       return(1);
+  
+#if USE_XRANDR  
+  if( XRRUpdateConfiguration_f && xevent.type == randrEventBase + RRScreenChangeNotify) {
+    XRRUpdateConfiguration_f(&xevent);
+    Fl::call_screen_init();
+    fl_init_workarea();
+    Fl::handle(FL_SCREEN_CONFIGURATION_CHANGED, NULL);
+  }
+#endif
 
+  if (xevent.type == PropertyNotify && xevent.xproperty.atom == fl_NET_WORKAREA) {
+    fl_init_workarea();
+  }
+  
   switch (xevent.type) {
 
   case KeymapNotify:
@@ -1234,7 +1301,7 @@ int fl_handle(const XEvent& thisevent)
     fl_key_vector[keycode/8] |= (1 << (keycode%8));
     static char *buffer = NULL;
     static int buffer_len = 0;
-    int len;
+    int len=0;
     KeySym keysym;
     if (buffer_len == 0) {
       buffer_len = 4096;
@@ -1242,7 +1309,7 @@ int fl_handle(const XEvent& thisevent)
     }
     if (xevent.type == KeyPress) {
       event = FL_KEYDOWN;
-      int len = 0;
+      len = 0;
 
       if (fl_xim_ic) {
 	Status status;
@@ -1434,18 +1501,25 @@ int fl_handle(const XEvent& thisevent)
     if (Fl::e_keysym == 0xfe20) Fl::e_keysym = FL_Tab;
 
     set_event_xy();
-    Fl::e_is_click = 0;
-    break;}
-
+    Fl::e_is_click = 0; }
+    break;
+  
   case ButtonPress:
     Fl::e_keysym = FL_Button + xevent.xbutton.button;
     set_event_xy();
+    Fl::e_dx = Fl::e_dy = 0;
     if (xevent.xbutton.button == Button4) {
       Fl::e_dy = -1; // Up
       event = FL_MOUSEWHEEL;
     } else if (xevent.xbutton.button == Button5) {
       Fl::e_dy = +1; // Down
       event = FL_MOUSEWHEEL;
+    } else if (xevent.xbutton.button == 6) {
+	Fl::e_dx = -1; // Left
+	event = FL_MOUSEWHEEL;
+    } else if (xevent.xbutton.button == 7) {
+	Fl::e_dx = +1; // Right
+	event = FL_MOUSEWHEEL;
     } else {
       Fl::e_state |= (FL_BUTTON1 << (xevent.xbutton.button-1));
       event = FL_PUSH;
@@ -1456,6 +1530,31 @@ int fl_handle(const XEvent& thisevent)
     in_a_window = true;
     break;
 
+  case PropertyNotify:
+    if (xevent.xproperty.atom == fl_NET_WM_STATE) {
+      int fullscreen_state = 0;
+      if (xevent.xproperty.state != PropertyDelete) {
+        unsigned long nitems;
+        unsigned long *words = 0;
+        if (0 == get_xwinprop(xid, fl_NET_WM_STATE, 64, &nitems, &words) ) { 
+          for (unsigned long item = 0; item < nitems; item++) {
+            if (words[item] == fl_NET_WM_STATE_FULLSCREEN) {
+              fullscreen_state = 1;
+            }
+          }
+        }
+      }
+      if (window->fullscreen_active() && !fullscreen_state) {
+        window->_clear_fullscreen();
+        event = FL_FULLSCREEN;
+      }
+      if (!window->fullscreen_active() && fullscreen_state) {
+        window->_set_fullscreen();
+        event = FL_FULLSCREEN;
+      }
+    }
+    break;
+
   case MotionNotify:
     set_event_xy();
 #  if CONSOLIDATE_MOTION
@@ -1564,14 +1663,13 @@ int fl_handle(const XEvent& thisevent)
 void Fl_Window::resize(int X,int Y,int W,int H) {
   int is_a_move = (X != x() || Y != y());
   int is_a_resize = (W != w() || H != h());
-  int is_a_enlarge = (W > w() || H > h());
   int resize_from_program = (this != resize_bug_fix);
   if (!resize_from_program) resize_bug_fix = 0;
   if (is_a_move && resize_from_program) set_flag(FORCE_POSITION);
   else if (!is_a_resize && !is_a_move) return;
   if (is_a_resize) {
     Fl_Group::resize(X,Y,W,H);
-    if (shown()) {redraw(); if(is_a_enlarge) i->wait_for_expose = 1;}
+    if (shown()) {redraw();}
   } else {
     x(X); y(Y);
   }
@@ -1595,6 +1693,75 @@ void Fl_Window::resize(int X,int Y,int W,int H) {
 
 ////////////////////////////////////////////////////////////////
 
+#define _NET_WM_STATE_REMOVE        0  /* remove/unset property */
+#define _NET_WM_STATE_ADD           1  /* add/set property */
+#define _NET_WM_STATE_TOGGLE        2  /* toggle property  */
+
+static void send_wm_state_event(Window wnd, int add, Atom prop) {
+  XEvent e;
+  e.xany.type = ClientMessage;
+  e.xany.window = wnd;
+  e.xclient.message_type = fl_NET_WM_STATE;
+  e.xclient.format = 32;
+  e.xclient.data.l[0] = add ? _NET_WM_STATE_ADD : _NET_WM_STATE_REMOVE;
+  e.xclient.data.l[1] = prop;
+  e.xclient.data.l[2] = 0;
+  e.xclient.data.l[3] = 0;
+  e.xclient.data.l[4] = 0;
+  XSendEvent(fl_display, RootWindow(fl_display, fl_screen),
+             0, SubstructureNotifyMask | SubstructureRedirectMask,
+             &e);
+}
+
+int Fl_X::ewmh_supported() {
+  static int result = -1;
+
+  if (result == -1) {
+    result = 0;
+    unsigned long nitems;
+    unsigned long *words = 0;
+    if (0 == get_xwinprop(XRootWindow(fl_display, fl_screen), fl_NET_SUPPORTING_WM_CHECK, 64,
+                          &nitems, &words) && nitems == 1) {
+      Window child = words[0];
+      if (0 == get_xwinprop(child, fl_NET_SUPPORTING_WM_CHECK, 64,
+                           &nitems, &words) && nitems == 1) {
+        result = (child == words[0]);
+      }
+    }
+  }
+
+  return result;
+}
+
+/* Change an existing window to fullscreen */
+void Fl_Window::fullscreen_x() {
+  if (Fl_X::ewmh_supported()) {
+    send_wm_state_event(fl_xid(this), 1, fl_NET_WM_STATE_FULLSCREEN);
+  } else {
+    _set_fullscreen();
+    hide();
+    show();
+    /* We want to grab the window, not a widget, so we cannot use Fl::grab */
+    XGrabKeyboard(fl_display, fl_xid(this), 1, GrabModeAsync, GrabModeAsync, fl_event_time);
+    Fl::handle(FL_FULLSCREEN, this);
+  }
+}
+
+void Fl_Window::fullscreen_off_x(int X, int Y, int W, int H) {
+  if (Fl_X::ewmh_supported()) {
+    send_wm_state_event(fl_xid(this), 0, fl_NET_WM_STATE_FULLSCREEN);
+  } else {
+    _clear_fullscreen();
+    /* The grab will be lost when the window is destroyed */
+    hide();
+    resize(X,Y,W,H);
+    show();
+    Fl::handle(FL_FULLSCREEN, this);
+  }
+}
+
+////////////////////////////////////////////////////////////////
+
 // A subclass of Fl_Window may call this to associate an X window it
 // creates with the Fl_Window:
 
@@ -1630,6 +1797,7 @@ ExposureMask|StructureNotifyMask
 |KeyPressMask|KeyReleaseMask|KeymapStateMask|FocusChangeMask
 |ButtonPressMask|ButtonReleaseMask
 |EnterWindowMask|LeaveWindowMask
+|PropertyChangeMask
 |PointerMotionMask;
 
 void Fl_X::make_xid(Fl_Window* win, XVisualInfo *visual, Colormap colormap)
@@ -1645,7 +1813,7 @@ void Fl_X::make_xid(Fl_Window* win, XVisualInfo *visual, Colormap colormap)
   if (!win->parent() && !Fl::grab()) {
     // center windows in case window manager does not do anything:
 #ifdef FL_CENTER_WINDOWS
-    if (!(win->flags() & Fl_Widget::FORCE_POSITION)) {
+    if (!win->force_position()) {
       win->x(X = scr_x+(scr_w-W)/2);
       win->y(Y = scr_y+(scr_h-H)/2);
     }
@@ -1701,6 +1869,16 @@ void Fl_X::make_xid(Fl_Window* win, XVisualInfo *visual, Colormap colormap)
     attr.save_under = 1; mask |= CWSaveUnder;
     if (!win->border()) {attr.override_redirect = 1; mask |= CWOverrideRedirect;}
   }
+  // For the non-EWMH fullscreen case, we cannot use the code above,
+  // since we do not want save_under, do not want to turn off the
+  // border, and cannot grab without an existing window. Besides, 
+  // there is no clear_override(). 
+  if (win->flags() & Fl_Widget::FULLSCREEN && !Fl_X::ewmh_supported()) {
+    attr.override_redirect = 1;
+    mask |= CWOverrideRedirect;
+    Fl::screen_xywh(X, Y, W, H, X, Y, W, H);
+  }
+
   if (fl_background_pixel >= 0) {
     attr.background_pixel = fl_background_pixel;
     fl_background_pixel = -1;
@@ -1732,16 +1910,17 @@ void Fl_X::make_xid(Fl_Window* win, XVisualInfo *visual, Colormap colormap)
     // set the class property, which controls the icon used:
     if (win->xclass()) {
       char buffer[1024];
-      char *p; const char *q;
-      // truncate on any punctuation, because they break XResource lookup:
-      for (p = buffer, q = win->xclass(); isalnum(*q)||(*q&128);) *p++ = *q++;
-      *p++ = 0;
+      const char *xclass = win->xclass();
+      const int len = strlen(xclass);
+      // duplicate the xclass string for use as XA_WM_CLASS
+      strcpy(buffer, xclass);
+      strcpy(buffer + len + 1, xclass);
       // create the capitalized version:
-      q = buffer;
-      *p = toupper(*q++); if (*p++ == 'X') *p++ = toupper(*q++);
-      while ((*p++ = *q++));
+      buffer[len + 1] = toupper(buffer[len + 1]);
+      if (buffer[len + 1] == 'X')
+        buffer[len + 2] = toupper(buffer[len + 2]);
       XChangeProperty(fl_display, xp->xid, XA_WM_CLASS, XA_STRING, 8, 0,
-                      (unsigned char *)buffer, p-buffer-1);
+                      (unsigned char *)buffer, len * 2 + 2);
     }
 
     if (win->non_modal() && xp->next && !fl_disable_transient_for) {
@@ -1760,6 +1939,12 @@ void Fl_X::make_xid(Fl_Window* win, XVisualInfo *visual, Colormap colormap)
           PropModeAppend, (unsigned char*) &net_wm_state_skip_taskbar, 1);
     }
 
+    // If asked for, create fullscreen
+    if (win->flags() & Fl_Widget::FULLSCREEN && Fl_X::ewmh_supported()) {
+      XChangeProperty (fl_display, xp->xid, fl_NET_WM_STATE, XA_ATOM, 32,
+                       PropModeAppend, (unsigned char*) &fl_NET_WM_STATE_FULLSCREEN, 1);
+    }
+
     // Make it receptive to DnD:
     long version = 4;
     XChangeProperty(fl_display, xp->xid, fl_XdndAware,
@@ -1797,6 +1982,12 @@ void Fl_X::make_xid(Fl_Window* win, XVisualInfo *visual, Colormap colormap)
     Fl::e_number = old_event;
     win->redraw();
   }
+
+  // non-EWMH fullscreen case, need grab
+  if (win->flags() & Fl_Widget::FULLSCREEN && !Fl_X::ewmh_supported()) {
+    XGrabKeyboard(fl_display, xp->xid, 1, GrabModeAsync, GrabModeAsync, fl_event_time);
+  }
+
 }
 
 ////////////////////////////////////////////////////////////////
@@ -1858,7 +2049,7 @@ void Fl_X::sendxjunk() {
     prop[1] = 1|2|16; // MWM_FUNC_ALL | MWM_FUNC_RESIZE | MWM_FUNC_MAXIMIZE
   }
 
-  if (w->flags() & Fl_Widget::FORCE_POSITION) {
+  if (w->force_position()) {
     hints->flags |= USPosition;
     hints->x = w->x();
     hints->y = w->y();
@@ -2110,5 +2301,5 @@ void preparePrintFront(void)
 #endif
 
 //
-// End of "$Id: Fl_x.cxx 8764 2011-05-30 16:47:48Z manolo $".
+// End of "$Id: Fl_x.cxx 9699 2012-10-16 15:35:34Z manolo $".
 //
