@@ -1,24 +1,15 @@
 //
-// "$Id: fl_font.cxx 8722 2011-05-23 16:06:13Z ianmacarthur $"
+// "$Id: fl_font.cxx 9325 2012-04-05 05:12:30Z fabien $"
 //
 // Font selection code for the Fast Light Tool Kit (FLTK).
 //
 // Copyright 1998-2010 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -66,20 +57,20 @@
 
 
 double fl_width(const char* c) {
-  if (c) return fl_width(c, strlen(c));
+  if (c) return fl_width(c, (int) strlen(c));
   else return 0.0f;
 }
 
 void fl_draw(const char* str, int x, int y) {
-  fl_draw(str, strlen(str), x, y);
+  fl_draw(str, (int) strlen(str), x, y);
 }
 
 void fl_draw(int angle, const char* str, int x, int y) {
-  fl_draw(angle, str, strlen(str), x, y);//must be fixed!
+  fl_draw(angle, str, (int) strlen(str), x, y);//must be fixed!
 }
 
 void fl_text_extents(const char *c, int &dx, int &dy, int &w, int &h) {
-  if (c)  fl_text_extents(c, strlen(c), dx, dy, w, h);
+  if (c)  fl_text_extents(c, (int) strlen(c), dx, dy, w, h);
   else {
     w = 0; h = 0;
     dx = 0; dy = 0;
@@ -95,5 +86,5 @@ void fl_draw(const char* str, int l, float x, float y) {
 #endif
 }
 //
-// End of "$Id: fl_font.cxx 8722 2011-05-23 16:06:13Z ianmacarthur $".
+// End of "$Id: fl_font.cxx 9325 2012-04-05 05:12:30Z fabien $".
 //
