@@ -1,24 +1,15 @@
 //
-// "$Id: fl_scroll_area.cxx 5714 2007-02-25 00:00:49Z matt $"
+// "$Id: fl_scroll_area.cxx 8864 2011-07-19 04:49:30Z greg.ercolano $"
 //
 // Scrolling routines for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -35,8 +26,19 @@
 #include <FL/fl_draw.H>
 
 // scroll a rectangle and redraw the newly exposed portions:
+/**
+  Scroll a rectangle and draw the newly exposed portions.
+  \param[in] X,Y       position of top-left of rectangle
+  \param[in] W,H       size of rectangle
+  \param[in] dx,dy     pixel offsets for shifting rectangle
+  \param[in] draw_area callback function to draw rectangular areas
+  \param[in] data      pointer to user data for callback
+  The contents of the rectangular area is first shifted by \p dx
+  and \p dy pixels. The \p draw_area callback is then called for
+  every newly exposed rectangular area.
+  */
 void fl_scroll(int X, int Y, int W, int H, int dx, int dy,
-	       void (*draw_area)(void*, int,int,int,int), void* data)
+               void (*draw_area)(void*, int,int,int,int), void* data)
 {
   if (!dx && !dy) return;
   if (dx <= -W || dx >= W || dy <= -H || dy >= H) {
@@ -72,7 +74,20 @@ void fl_scroll(int X, int Y, int W, int H, int dx, int dy,
     clip_y = Y+src_h;
     clip_h = H-src_h;
   }
-#ifdef WIN32
+
+#if defined(USE_X11)
+  XCopyArea(fl_display, fl_window, fl_window, fl_gc,
+	    src_x, src_y, src_w, src_h, dest_x, dest_y);
+  // we have to sync the display and get the GraphicsExpose events! (sigh)
+  for (;;) {
+    XEvent e; XWindowEvent(fl_display, fl_window, ExposureMask, &e);
+    if (e.type == NoExpose) break;
+    // otherwise assume it is a GraphicsExpose event:
+    draw_area(data, e.xexpose.x, e.xexpose.y,
+	      e.xexpose.width, e.xexpose.height);
+    if (!e.xgraphicsexpose.count) break;
+  }
+#elif defined(WIN32)
   typedef int (WINAPI* fl_GetRandomRgn_func)(HDC, HRGN, INT);
   static fl_GetRandomRgn_func fl_GetRandomRgn = 0L;
   static char first_time = 1;
@@ -123,44 +138,23 @@ void fl_scroll(int X, int Y, int W, int H, int dx, int dy,
     }
   }
 
-  // Great, we can do an accelerated scroll insteasd of re-rendering
+  // Great, we can do an accelerated scroll instead of re-rendering
   BitBlt(fl_gc, dest_x, dest_y, src_w, src_h, fl_gc, src_x, src_y,SRCCOPY);
 
-#elif defined(__APPLE_QD__)
-  Rect src = { src_y, src_x, src_y+src_h, src_x+src_w };
-  Rect dst = { dest_y, dest_x, dest_y+src_h, dest_x+src_w };
-  static RGBColor bg = { 0xffff, 0xffff, 0xffff }; RGBBackColor( &bg );
-  static RGBColor fg = { 0x0000, 0x0000, 0x0000 }; RGBForeColor( &fg );
-  CopyBits( GetPortBitMapForCopyBits( GetWindowPort(fl_window) ),
-            GetPortBitMapForCopyBits( GetWindowPort(fl_window) ), &src, &dst, srcCopy, 0L);
 #elif defined(__APPLE_QUARTZ__)
-  // warning: there does not seem to be an equivalent to this function in Quartz
-  // ScrollWindowRect is a QuickDraw function and won't work here.
-  // Since on OS X all windows are fully double buffered, we need not
-  // worry about offscreen or obscured areas
-  Rect src = { src_y, src_x, src_y+src_h, src_x+src_w };
-  Rect dst = { dest_y, dest_x, dest_y+src_h, dest_x+src_w };
-  static RGBColor bg = { 0xffff, 0xffff, 0xffff }; RGBBackColor( &bg );
-  static RGBColor fg = { 0x0000, 0x0000, 0x0000 }; RGBForeColor( &fg );
-  CopyBits( GetPortBitMapForCopyBits( GetWindowPort(fl_window) ),
-            GetPortBitMapForCopyBits( GetWindowPort(fl_window) ), &src, &dst, srcCopy, 0L);
+  CGImageRef img = Fl_X::CGImage_from_window_rect(Fl_Window::current(), src_x, src_y, src_w, src_h);
+  CGRect rect = { { dest_x, dest_y }, { src_w, src_h } };
+  Fl_X::q_begin_image(rect, 0, 0, src_w, src_h);
+  CGContextDrawImage(fl_gc, rect, img);
+  Fl_X::q_end_image();
+  CFRelease(img);
 #else
-  XCopyArea(fl_display, fl_window, fl_window, fl_gc,
-	    src_x, src_y, src_w, src_h, dest_x, dest_y);
-  // we have to sync the display and get the GraphicsExpose events! (sigh)
-  for (;;) {
-    XEvent e; XWindowEvent(fl_display, fl_window, ExposureMask, &e);
-    if (e.type == NoExpose) break;
-    // otherwise assumme it is a GraphicsExpose event:
-    draw_area(data, e.xexpose.x, e.xexpose.y,
-	      e.xexpose.width, e.xexpose.height);
-    if (!e.xgraphicsexpose.count) break;
-  }
+# error unsupported platform
 #endif
   if (dx) draw_area(data, clip_x, dest_y, clip_w, src_h);
   if (dy) draw_area(data, X, clip_y, W, clip_h);
 }
 
 //
-// End of "$Id: fl_scroll_area.cxx 5714 2007-02-25 00:00:49Z matt $".
+// End of "$Id: fl_scroll_area.cxx 8864 2011-07-19 04:49:30Z greg.ercolano $".
 //
