@@ -1,24 +1,15 @@
 /*
- * "$Id: flstring.h 5701 2007-02-20 18:43:10Z mike $"
+ * "$Id: flstring.h 9573 2012-06-06 03:38:02Z fabien $"
  *
  * Common string header file for the Fast Light Tool Kit (FLTK).
  *
- * Copyright 1998-2007 by Bill Spitzak and others.
+ * Copyright 1998-2010 by Bill Spitzak and others.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * This library is free software. Distribution and use rights are outlined in
+ * the file "COPYING" which should have been included with this file.  If this
+ * file is missing or damaged, see the license at:
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
- *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA.
+ *     http://www.fltk.org/COPYING.php
  *
  * Please report all bugs and problems on the following page:
  *
@@ -51,13 +42,14 @@
 #  if defined(WIN32) && !defined(__CYGWIN__)
 #    define strcasecmp(s,t)	_stricmp((s), (t))
 #    define strncasecmp(s,t,n)	_strnicmp((s), (t), (n))
-// Visual C++ 2005 incorrectly displays a warning about the use of POSIX APIs
-// on Windows, which is supposed to be POSIX compliant...  Some of these
-// functions are also defined in ISO C99...
+/* Visual C++ 2005 incorrectly displays a warning about the use of POSIX APIs
+ * on Windows, which is supposed to be POSIX compliant...  Some of these
+ * functions are also defined in ISO C99...
+ */
 #    ifndef __WATCOMC__
 #      define strdup _strdup
 #      define unlink _unlink
-#    endif // !__WATCOMC__
+#    endif /* !__WATCOMC__ */
 #  elif defined(__EMX__)
 #    define strcasecmp(s,t)	stricmp((s), (t))
 #    define strncasecmp(s,t,n)	strnicmp((s), (t), (n))
@@ -67,18 +59,6 @@
 extern "C" {
 #  endif /* __cplusplus */
 
-/*
- * MetroWerks' CodeWarrior put thes "non-standard" functions in
- * <extras.h> which unfortunatly does not play well otherwise
- * when included - to be resolved...
- */
-
-#  if defined(__APPLE__) && defined(__MWERKS__) && defined(_MSL_USING_MW_C_HEADERS)
-int strcasecmp(const char*,const char*);
-int strncasecmp(const char*,const char*,int);
-char *strdup(const char*);
-#  endif
-
 FL_EXPORT extern int fl_snprintf(char *, size_t, const char *, ...);
 #  if !HAVE_SNPRINTF
 #    define snprintf fl_snprintf
@@ -104,12 +84,18 @@ FL_EXPORT extern size_t fl_strlcpy(char *, const char *, size_t);
 #    define strlcpy fl_strlcpy
 #  endif /* !HAVE_STRLCPY */
 
+/*
+ * locale independent ascii compare, does not introduce locale 
+ * pbs as w/  case cmp 
+ */
+FL_EXPORT extern int fl_ascii_strcasecmp(const char *s, const char *t);
+
 #  ifdef __cplusplus
 }
 #  endif /* __cplusplus */
+
 #endif /* !flstring_h */
 
-
 /*
- * End of "$Id: flstring.h 5701 2007-02-20 18:43:10Z mike $".
+ * End of "$Id: flstring.h 9573 2012-06-06 03:38:02Z fabien $".
  */
