@@ -1,31 +1,22 @@
 //
-// "$Id: forms_compatability.cxx 5190 2006-06-09 16:16:34Z mike $"
+// "$Id: forms_compatability.cxx 8864 2011-07-19 04:49:30Z greg.ercolano $"
 //
 // Forms compatibility functions for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
 //     http://www.fltk.org/str.php
 //
 
-// Forms library compatability functions.
+// Forms library compatibility functions.
 // Many more functions are defined as inlines in forms.h!
 
 #include <FL/forms.H>
@@ -138,11 +129,13 @@ Fl_Widget *fl_check_forms() {
 
 void fl_set_graphics_mode(int /*r*/,int /*d*/) {}
 
+#ifndef FL_DOXYGEN // FIXME: suppress doxygen warning
 void Fl_FormsText::draw() {
   draw_box();
-  align(align()|FL_ALIGN_INSIDE); // questionable method of compatability
+  align(align()|FL_ALIGN_INSIDE); // questionable method of compatibility
   draw_label();
 }
+#endif
 
 // Create a forms button by selecting correct fltk subclass:
 
@@ -202,10 +195,10 @@ int fl_show_choice(
 }
 
 char *fl_show_simple_input(const char *str1, const char *defstr) {
-  const char *r = fl_input(str1, defstr);
+  const char *r = fl_input("%s", defstr, str1);
   return (char *)(r ? r : defstr);
 }
 
 //
-// End of "$Id: forms_compatability.cxx 5190 2006-06-09 16:16:34Z mike $".
+// End of "$Id: forms_compatability.cxx 8864 2011-07-19 04:49:30Z greg.ercolano $".
 //
