@@ -1,24 +1,15 @@
 //
-// "$Id: browser.cxx 5845 2007-05-20 00:01:06Z mike $"
+// "$Id: browser.cxx 9736 2012-12-05 14:53:03Z manolo $"
 //
 // Browser test program for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -77,7 +68,8 @@ Fl_Button	*top,
 		*bottom,
 		*middle,
 		*visible,
-		*swap;
+		*swap,
+		*sort;
 Fl_Int_Input	*field;
 
 void b_cb(Fl_Widget* o, void*) {
@@ -117,12 +109,16 @@ void swap_cb(Fl_Widget *, void *) {
   browser->swap(a, b);				// swap them
 }
 
+void sort_cb(Fl_Widget *, void *) {
+  browser->sort(FL_SORT_ASCENDING);
+}
+
 int main(int argc, char **argv) {
   int i;
   if (!Fl::args(argc,argv,i)) Fl::fatal(Fl::help);
   const char* fname = (i < argc) ? argv[i] : "browser.cxx";
-  Fl_Window window(400,400,fname);
-  browser = new Fl_Select_Browser(0,0,400,350,0);
+  Fl_Double_Window window(480,400,fname);
+  browser = new Fl_Select_Browser(0,0,480,350,0);
   browser->type(FL_MULTI_BROWSER);
   //browser->type(FL_HOLD_BROWSER);
   //browser->color(42);
@@ -130,29 +126,40 @@ int main(int argc, char **argv) {
   // browser->scrollbar_right();
   //browser->has_scrollbar(Fl_Browser::BOTH_ALWAYS);
   if (!browser->load(fname)) {
+    int done = 0;
 #ifdef _MSC_VER
     // if 'browser' was started from the VisualC environment in Win32, 
     // the current directory is set to the environment itself, 
     // so we need to correct the browser file path
-    int done = 1;
     if ( i == argc ) 
     {
       fname = "../test/browser.cxx";
       done = browser->load(fname);
     }
+#elif defined(__APPLE__)
+    if ( i == argc ) 
+    {
+      char buf[2048];
+      strcpy(buf, argv[0]);
+      char *slash = strrchr(buf, '/');
+      if (slash)
+#if defined(USING_XCODE)
+        strcpy(slash, "/../Resources/browser.cxx");
+#else
+	strcpy(slash, "/../../../browser.cxx");
+#endif
+      done = browser->load(buf);
+    }
+#endif
     if ( !done )
     {
-      printf("Can't load %s, %s\n", fname, strerror(errno));
+      fl_message("Can't load %s, %s\n", fname, strerror(errno));
       exit(1);
     }
-#else
-    printf("Can't load %s, %s\n", fname, strerror(errno));
-    exit(1);
-#endif
   }
   browser->position(0);
 
-  field = new Fl_Int_Input(50, 350, 350, 25, "Line #:");
+  field = new Fl_Int_Input(50, 350, 430, 25, "Line #:");
   field->callback(show_cb);
 
   top = new Fl_Button(0, 375, 80, 25, "Top");
@@ -171,12 +178,15 @@ int main(int argc, char **argv) {
   swap->callback(swap_cb);
   swap->tooltip("Swaps two selected lines\n(Use CTRL-click to select two lines)");
 
+  sort = new Fl_Button(400, 375, 80, 25, "Sort");
+  sort->callback(sort_cb);
+
   window.resizable(browser);
   window.show(argc,argv);
   return Fl::run();
 }
 
 //
-// End of "$Id: browser.cxx 5845 2007-05-20 00:01:06Z mike $".
+// End of "$Id: browser.cxx 9736 2012-12-05 14:53:03Z manolo $".
 //
 
