@@ -1,24 +1,15 @@
 //
-// "$Id: cursor.cxx 4288 2005-04-16 00:13:17Z mike $"
+// "$Id: cursor.cxx 8864 2011-07-19 04:49:30Z greg.ercolano $"
 //
 // Cursor test program for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -26,7 +17,7 @@
 //
 
 #include <FL/Fl.H>
-#include <FL/Fl_Window.H>
+#include <FL/Fl_Double_Window.H>
 #include <FL/Fl_Hor_Value_Slider.H>
 #include <FL/Fl_Choice.H>
 #include <FL/fl_draw.H>
@@ -39,7 +30,7 @@ Fl_Cursor cursor = FL_CURSOR_DEFAULT;
 Fl_Hor_Value_Slider *cursor_slider;
 
 void choice_cb(Fl_Widget *, void *v) {
-  cursor = (Fl_Cursor)(int)v;
+  cursor = (Fl_Cursor)(fl_intptr_t)v;
   cursor_slider->value(cursor);
   fl_cursor(cursor,fg,bg);
 }
@@ -100,7 +91,7 @@ public:
 };
 
 int main(int argc, char **argv) {
-  Fl_Window window(400,300);
+  Fl_Double_Window window(400,300);
 
   Fl_Choice choice(80,100,200,25,"Cursor:");
   choice.menu(choices);
@@ -161,5 +152,5 @@ int main(int argc, char **argv) {
 }
 
 //
-// End of "$Id: cursor.cxx 4288 2005-04-16 00:13:17Z mike $".
+// End of "$Id: cursor.cxx 8864 2011-07-19 04:49:30Z greg.ercolano $".
 //
